/*
 * arch/arm/mach-cxd4105/core.c
 *
 * machine_desc and initialize function
 *
 * Copyright 2007 Sony Corporation
 *
 * This code is based on arch/arm/mach-integrator/core.c.
 */
/*
 *  linux/arch/arm/mach-integrator/core.c
 *
 *  Copyright (C) 2000-2003 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2, as
 * published by the Free Software Foundation.
 */
#include <linux/device.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/sysdev.h>
#include <linux/mman.h>

#ifdef CONFIG_PM
#include <asm/arch/pm.h>
#endif /* CONFIG_PM */

#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/mach/map.h>
#include <asm/hardware.h>

#include <asm/io.h>
#include <asm/irq.h>

#include <asm/dma.h>
#include <asm/mach/dma.h>
#include <asm/arch/system.h>
#include <asm/arch/regs-dmac.h>
#include <asm/arch/regs-timer.h>
#include <asm/arch/regs-clk.h>
#include <asm/arch/regs-emc.h>

/*---------------------------- I/O mapping --------------------------*/
/*
  Logical    Physical
  C0000000   30000000 SDRAM (EMC CS1)
  
  E0000000   70000000 Peripherals (AHB)
  E0C00000   -
  E2000000   72000000 Peripherals (APB)
  Ea000000   -

  FFFF0000            High Vector (arch/mm/mm-armv.c)
  FFFF2000   FFFF2000 ETM(8KB)
*/
 
static struct map_desc cxd4105_io_desc[] __initdata = {
	{
		.virtual	= IO_ADDRESS(CXD4105_AHB_BASE),
		.pfn		= __phys_to_pfn(CXD4105_AHB_BASE),
		.length		= CXD4105_AHB_SIZE,
		.type		= MT_DEVICE },
	{
		.virtual	= IO_ADDRESS(CXD4105_APB_BASE),
		.pfn		= __phys_to_pfn(CXD4105_APB_BASE),
		.length		= CXD4105_APB_SIZE,
		.type		= MT_DEVICE },
#if !defined(CONFIG_ARCH_MB8AC102)
	{
		.virtual	= HIO_ADDRESS(CXD4105_HIO_BASE),
		.pfn		= __phys_to_pfn(CXD4105_HIO_BASE),
		.length		= CXD4105_HIO_SIZE,
		.type		= MT_DEVICE },
#endif /* !CONFIG_ARCH_MB8AC102 */
};

void __init cxd4105_map_io(void)
{
	iotable_init(cxd4105_io_desc, ARRAY_SIZE(cxd4105_io_desc));
}

/*------------------------------ SYSDEV -----------------------------*/
#ifdef CONFIG_PM
static struct {
	int valid;
	u32 intc;
	u32 emc_bind;
	u32 emc_am2_cfg;
	u32 emc_am2_prm1;
	u32 emc_am2_prm2;
} sv = { 
	.valid = 0,
};

static int cxd4105_suspend(struct sys_device *dev, pm_message_t state)
{
	/* Save INTC */
	sv.intc = readl(INT_CTRL(IRQ_ENABLE));
	writel(0xffffffff, INT_CTRL(IRQ_ENABLE_CLEAR));

	/* Save EMC CS2 */
	sv.emc_bind = readl(CXD4105_EMC_VIRT_BASE+EMC_CS_PHY_BIND);
	sv.emc_am2_cfg = readl(CXD4105_EMC_VIRT_BASE+EMC_PHY_AM2_CFG);
	sv.emc_am2_prm1 = readl(CXD4105_EMC_VIRT_BASE+EMC_PHY_AM2_PRM1);
	sv.emc_am2_prm2 = readl(CXD4105_EMC_VIRT_BASE+EMC_PHY_AM2_PRM2);

	sv.valid = 1;
	return 0;
}

static int cxd4105_resume(struct sys_device *dev)
{
	/* Enable DMAC */
	writel(0x1, DMACCONFIGRATION);

	if (sv.valid) {
		/* Restore EMC CS2 */
		writel(sv.emc_bind, CXD4105_EMC_VIRT_BASE+EMC_CS_PHY_BIND);
		writel(sv.emc_am2_cfg, CXD4105_EMC_VIRT_BASE+EMC_PHY_AM2_CFG);
		writel(sv.emc_am2_prm1, CXD4105_EMC_VIRT_BASE+EMC_PHY_AM2_PRM1);
		writel(sv.emc_am2_prm2, CXD4105_EMC_VIRT_BASE+EMC_PHY_AM2_PRM2);

		/* Restore INTC */
		writel(0xffffffff, INT_CTRL(IRQ_ENABLE_CLEAR));
		writel(sv.intc, INT_CTRL(IRQ_ENABLE));

		sv.valid = 0;
	}
	return 0;
}
#endif /* CONFIG_PM */

static struct sysdev_class cxd4105_sysclass = {
	set_kset_name("cxd4105"),
#ifdef CONFIG_PM
	.suspend	= cxd4105_suspend,
	.resume		= cxd4105_resume,
#endif
};

static struct sys_device cxd4105_device = {
	.id		= 0,
	.cls		= &cxd4105_sysclass,
};

int __init cxd4105_core_init(void)
{
	int ret;
#ifdef CONFIG_OVERCOMMIT_ALWAYS
	extern int sysctl_overcommit_memory;
#endif

#ifdef CONFIG_PM
	pm_power_off = cxd4105_poweroff;
#endif
#ifdef CONFIG_OVERCOMMIT_ALWAYS
	sysctl_overcommit_memory = OVERCOMMIT_ALWAYS;
#endif

	/* enable DMAC0 */
	writel(0x1, DMACCONFIGRATION);

	ret = sysdev_class_register(&cxd4105_sysclass);
	if (ret) {
		printk(KERN_ERR "cxd4105_core_init: sysdev_class_register failed.\n");
		return ret;
	}
	ret = sysdev_register(&cxd4105_device);
	if (ret) {
		printk(KERN_ERR "cxd4105_core_init: sysdev_register failed.\n");
		return ret;
	}
	return ret;
}

#ifdef CONFIG_MEMORY_HOTPLUG
unsigned int target_get_mem_type(unsigned long phys)
{
	return MT_HOTPLUG;
}
#endif
