/*
 * arch/arm/mach-cxd4105/sleep4108.S
 *
 * pm callback for cxd4108
 *
 * Copyright 2004,2005,2006,2007,2008 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/sleep.S
 */
/*
 * SA11x0 Assembler Sleep/WakeUp Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06: Cliff Brake         Initial code
 *
 * 2001-08-29:	Nicolas Pitre	Simplified.
 *
 * 2002-05-27:	Nicolas Pitre	Revisited, more cleanup and simplification.
 *				Storage is on the stack now.
 */

#include <linux/linkage.h>
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <asm/arch/regs-uart.h>
#include <asm/arch/regs-emc.h>
#include <asm/arch/regs-sdc.h>
#include <asm/arch/regs-clk.h>
#include <asm/arch/regs-gpio.h>
#include <asm/arch/debug-macro.S>

#include <asm/arch/pm.h>

/*
	resume sram base
*/
#define RESUMESRAM_BASE (VA_ETB)
#define RESUMESRAM_PHYS (ETB_BASE)

/* below is used in system MMU off */
/*
	SDRAM control region base
*/
#define EMC_BASE CXD4105_EMC_BASE
/*
	DDR contorl region base
*/
#define SDC_BASE CXD4105_SDC_BASE
/*
	clk control region base
*/
#define CLK_BASE (VA_CLKBLK_BASE+CLKBLK_PSCKETB)
#define		PSCKETB (0x0)
#define		RSTETB	(CLKBLK_RSTETB-CLKBLK_PSCKETB)
		
#define SHARED_MEM	0x200FFFF8
#define		ERROR	0
#define		STATUS  1
#define		SUSCMD	2
#define		PWRKEY  3
							
		.data


/*
 * cxd4105_cpu_suspend()
 *
 * Causes cxd4105 to enter sleep state
 *
 */

ENTRY(cxd4105_cpu_suspend)
	@ r0: mode flag

	stmfd	sp!, {r4 - r12, lr}		@ save registers on stack
	str	r0, modeflag			@ save mode flag

	@ get coprocessor registers
	mrc 	p15, 0, r4, c3, c0, 0		@ domain ID
	mrc 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mrc	p15, 0, r6, c13, c0, 0		@ PID
	mrc 	p15, 0, r7, c1, c0, 0		@ control reg
	
	@ store them plus current virtual stack ptr on stack
	mov	r8, sp
	stmfd	sp!, {r4 - r8}

	@ preserve phys address of stack
	mov	r0, sp
	bl	sleep_phys_sp
	ldr	r1, =sleep_save_sp
	str	r0, [r1]
	
	@ disable interrupt 
	mov	r0, #PSR_F_BIT | PSR_I_BIT | SVC_MODE
	msr	cpsr_c, r0

	@ clean & invalide D&I cache, necessary?
	bl	arm926_flush_kern_cache_all

	ldr	r0, modeflag
	cmp	r0, #PM_ENTER_RESET
	beq	goto_suspend

	@ for creating Warm-Boot-Image
	bl	cxd4105_create_warmbootimage

	cmp	r0, #0
	beq	goto_suspend

	/*
		failure handling for warm_boot_image
	*/
	mov	r0, sp
	ldmfd	r0, {r4 - r7, sp}
	mcr 	p15, 0, r4, c3, c0, 0		@ domain ID
	mcr 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mcr	p15, 0, r6, c13, c0, 0		@ PID
	mcr 	p15, 0, r7, c1, c0, 0		@ page table
	ldmfd	sp!, {r4 - r12, pc}

	/*
		sdram refresh - preparation
	*/
goto_suspend:
#ifdef CONFIG_SNSC_DEBUG_PROFILE
	adr	r0, __printascii
	bl	profile_dump_record
#endif
	ldr	r0, =CLK_BASE
	mov	r1, #0x00
	str	r1, [r0, #PSCKETB]		@ disable PSCKETB

	@ wait for >1us
	mov	r0, #2
	bl	__udelay

	@ enable ETB 
	ldr	r0, =CLK_BASE
	mov	r1, #0x01
	str	r1, [r0, #RSTETB]		@ enable RESET CLEAR ETB

	@ wait for >1us
	mov	r0, #2
	bl	__udelay

	ldr	r0, =CLK_BASE
	mov	r1, #0x01
	str	r1, [r0, #PSCKETB]		@ enable PSCKETB

	@ wait for >1us
	mov	r0, #2
	bl	__udelay			

	@ copy the standby operation code to the resume ram
	ldr	r0, =RESUMESRAM_BASE
	adr	r1, exec_in_resume_ram
	adr	r2, exec_in_resume_ram_end

code_copy:		
	ldr	r3, [r1], #4
	str	r3, [r0], #4
	teq	r1, r2
	bne	code_copy

	@ r4-r11 are for control region base address
	@ load neccesary data to register to avoid accessing sdr/ddr
	ldr	r4, =SDC_BASE
	ldr	r5, =EMC_BASE
	ldr	r6, =SHARED_MEM
	ldr	r7, resume_vector_addr		@ address of the end of DDR	
	@ physical address of cxd4105_cpu_resume
	ldr	r0, =cxd4105_cpu_resume
	bl	sleep_phys_sp
	mov	r8, r0				@ resume vector
	bl	__printhex8
	mov	r0, #':'
	bl	__printch
	@ resume ram base address
	ldr	r9, =RESUMESRAM_PHYS		@ Physical address
	@ resume_vector_backup phys addr
	ldr	r0, =resume_vector_backup
	bl	sleep_phys_sp
	mov	r10, r0				
	@ loops per jiffy
	ldr	r0, LC0				
	ldr	r11, [r0]
	
	@ clean & invalide D&I cache
	bl	arm926_flush_kern_cache_all
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x1000			@ ...i............
	bic	r0, r0, #0x000e			@ ............wca.
	mcr	p15, 0, r0, c1, c0, 0		@ disable caches
 	@ disable MMU, the mapping is flat so
 	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
 	bic	r0, r0, #0x0001			@ ...............m
 	mcr	p15, 0, r0, c1, c0, 0		@ disable MMU
	@ jump to the resume ram(Physical address)
	mov	pc, r9				@ fetch: MMU active
	nop					@ fetch: MMU active
	nop

	.ltorg
	.align	5
exec_in_resume_ram:
	@ Invert STABLE_RUN for DEBUG
	ldr	r0, =CXD4105_MISCCTRL(MISCCTRL_STABLERUN)
	ldr	r1, [r0, #MISCCTRL_DATA]
	cmp	r1, #0
	mov	r1, #1
	streq	r1, [r0, #MISCCTRL_SET]		@ SET
	strne	r1, [r0, #MISCCTRL_CLR]		@ CLR

	@ save resume vector
	ldr	r0, [r7]			@ load original data
	str	r0, [r10]			@ save to resume_vector_backup
	str	r8, [r7]			@ save resume vector
	
	mov	r0, #0
	strb	r0, [r6, #ERROR]
	strb	r0, [r6, #STATUS]
	strb	r0, [r6, #SUSCMD]

	@ force RESET ?
	ldr	r0, modeflag
	cmp	r0, #PM_ENTER_RESET
	beq	stop_clk
	
	mov	r0, #'\n'
	bl	__printch
	ldr	r0, [r6]
	bl	__printhex8
	mov	r0, #'-'
	bl	__printch

	mov	r0, #1
	strb	r0, [r6, #SUSCMD]
retry:	ldrb	r0, [r6, #STATUS]
	cmp	r0, #0
	beq	retry

	ldr	r0, [r6]
	bl	__printhex8
	mov	r0, #'\n'
	bl	__printch

stop_clk:
	@ stop
	ldr	r1, =CXD4105_CLKBLK_BASE
	mov	r0, #0
	str	r0, [r1, #CLKBLK_PSCKAARM]	@ clk stop

	mov	r0, #1
	bl	__udelay_r

	ldr	r1, =CXD4105_CLKBLK_BASE
	mov	r0, #0
	str	r0, [r1, #CLKBLK_AARMSRST]	@ reset

	ldr	r1, modeflag
	cmp	r1, #PM_ENTER_NORMAL
	@ if !PM_ENTER_NORMAL, set ERROR flag for Loader
	movne	r0, #1
	strneb	r0, [r6, #ERROR]
	cmp	r1, #PM_ENTER_RESET
	beq	set_reboot

	ldrb	r0, [r6, #PWRKEY]		@ Power Key Status
	cmp	r0, #0				@ if reboot?
	beq	set_reboot_end
set_reboot:
	mov	r0, #1
	str	r0, reboot
set_reboot_end:

	@ set DDR to self-refresh
#if 1 /* SDC bug work around */
	@ REFMD
	ldr	r0, [r4, #SD_PARA_4]
	bic	r0, r0, #0xFF00
	bic	r0, r0, #0x00F0
	orr	r0, r0, #0x130			
	str	r0, [r4, #SD_PARA_4]		@ 15-4 = 0x13
	@ wait for >39us
	mov	r0, #40
	bl	__udelay_r			
	@ CLK24
	ldr	r0, =0x79a00000			@ HTP
	ldr	r1, [r0]
	bic	r1, r1, #1<<18			@ XPS_SDC_CK24 off
	str	r1, [r0]
	@ wait for 610ns~39us	
	mov	r0, #3
	bl	__udelay_r
#endif /* SDC bug work around */
ddr_sref:		
	ldr	r0, [r4, #SD_PARA_4]
	bic	r0, r0, #(1 << 25 | 1 << 26)    @ clear SRFEX & SREET
	str	r0, [r4, #SD_PARA_4]
	
	orr	r0, r0, #1 << 26                @ set SREET
	str	r0, [r4, #SD_PARA_4]

	mov	r1, #0x10000
10:	cmp	r1, #0
	beq	ddr_sref
	ldr	r0, [r4, #SD_PARA_4]
	ands	r0, r0, #1 << 30                @ SRF_IN
	subeq	r1, r1, #1
	beq	10b	

#if defined(CONFIG_CXD4105_USE_SDR)
        @ set SDRAM Self-Refresh
	@ Flush FIFOs
	mov	r0, #0x07
	str	r0, [r5, #EMC_TRANS_CTRL]
	
10:	ldr	r0, [r5, #EMC_TRANS_CTRL]
	cmp	r0, #0
	bne	10b
	
        mov	r0, #0x8000
	@ DPD mode seems not work
#if defined(CONFIG_CXD4105_DPD_MODE_dummy)	
	orr	r0, r0, #0x0C			@ DPD
#else
	orr	r0, r0, #0x03			@ SREF
#endif	
        str	r0, [r5, #EMC_PHY_SDR_CMD]
	@ delay > 1ms for sdr SREF
	mov	r0, #0x400
	bl	__udelay_r
#endif

	mov	r0, #'E'
	bl	__printch
	mov	r0, #'\r'
	bl	__printch
	mov	r0, #'\n'
	bl	__printch

	/*
	 * D-D Conv. control
	 */
	ldr	r0, reboot
	cmp	r0, #1
	beq	dd_reboot

dd_off:	
	ldr	r4, =CXD4105_GPIO(GPIO_POWEROFF)
	mov	r5, #XPOWER_OFF
	str	r5, [r4, #GPIO_DIR]		@ DIR=OUT
	str	r5, [r4, #GPIO_DATA_CLEAR]	@ CLR (assert)
	b	end_job

dd_reboot:
	ldr	r4, =CXD4105_MISCCTRL(MISCCTRL_XRESETREQ)
	mov	r5, #1
	str	r5, [r4, #MISCCTRL_CLR]		@ CLR (assert)

end_job:
	@ wait for interrupt
	mov	r0, #0
	mcr	p15, 0, r0, c7, c0, 4
20:	b	20b			@ loop waiting for sleep


LC0:		.word	loops_per_jiffy

/* copy from arch/arm/lib/delay.S
 * 0 <= r0 <= 2000
 */
__udelay_r:	
		ldr	r2, =(0x68db/2)		@ takes more time on resume ram
		mul	r0, r2, r0

		mov	r2, r11
		mov	r0, r0, lsr #11		@ max = 0x00003fff
		mov	r2, r2, lsr #11		@ max = 0x0003ffff
		mul	r0, r2, r0		@ max = 2^32-1
		movs	r0, r0, lsr #6
		moveq	pc,lr
1:		subs	r0, r0, #1
		bhi	1b
		mov	pc,lr

modeflag:	.long	0x00
reboot:		.long	0x00

#if defined(CONFIG_DEBUG_LL)	
	@ for debug
__printhex8:	
	mov	r1, #8
	b	__printhex

__printhex4:	
	mov	r1, #4
	b	__printhex

__printhex2:	
	mov	r1, #2
__printhex:
	adr	r2, hexbuf
	add	r3, r2, r1
	mov	r1, #0
	strb	r1, [r3]
1:	and	r1, r0, #15
	mov	r0, r0, lsr #4
	cmp	r1, #10
	addlt	r1, r1, #'0'
	addge	r1, r1, #'a' - 10
	strb	r1, [r3, #-1]!
	teq	r3, r2
	bne	1b
	mov	r0, r2
	b	__printascii

__printascii:	
	addruart r3
	b	2f
1:	waituart r2, r3
	senduart r1, r3
	busyuart r2, r3
	teq	r1, #'\n'
	moveq	r1, #'\r'
	beq	1b
2:	teq	r0, #0
	ldrneb	r1, [r0], #1
	teqne	r1, #0
	bne	1b
	mov	pc, lr

__printch:	
	addruart r3
	mov	r1, r0
	mov	r0, #0
	b	1b

hexbuf:		.space 16
#else
__printhex8:
	mov	pc, lr
__printhex4:
	mov	pc, lr
__printhex2:
	mov	pc, lr
__printhex:
	mov	pc, lr
__printascii:
	mov	pc, lr
__printch:
	mov	pc, lr
#endif
	.ltorg
exec_in_resume_ram_end:

/*
 * cxd4105_cpu_resume()
 *
 * entry point from bootloader into kernel during resume
 *   Redboot does not have resume vector.
 *   Please jump to this vector by "go" command of RedBoot.
 */

	.align 5
ENTRY(cxd4105_cpu_resume)
	mov	r0, #PSR_F_BIT | PSR_I_BIT | SVC_MODE
	msr	cpsr_c, r0			@ set SVC, irqs off

	mov	r0, #'R'
	bl	__printch
	
tc_loop:mrc	p15, 0, r15, c7, c10, 3		@ test & clean D cache
	bne	tc_loop
	
	mov	r0, #0
	mcr	p15, 0, r0, c7, c7, 0		@ invalidate I,D caches
	mcr	p15, 0, r0, c7, c10, 4		@ drain WB
	mcr	p15, 0, r0, c8, c7, 0		@ invalidate I & D TLBs
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x000f			@ ............wcam
	bic	r0, r0, #0x1100			@ ...i...s........
	mcr	p15, 0, r0, c1, c0, 0		@ ctrl register
	nop
	nop

	@ restore data of resume vector
	ldr	r0, resume_vector_backup
	ldr	r1, resume_vector_addr
	str	r0, [r1]
 
	ldr	r0, sleep_save_sp		@ stack phys addr
	ldr	r2, =resume_after_mmu		@ its absolute virtual address
	ldmfd	r0, {r4 - r7, sp}		@ CP regs + virt stack ptr

	mcr 	p15, 0, r4, c3, c0, 0		@ domain ID
	mcr 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mcr	p15, 0, r6, c13, c0, 0		@ PID

	b	resume_turn_on_mmu		@ cache align execution

	.align 5
resume_turn_on_mmu:
	mcr 	p15, 0, r7, c1, c0, 0		@ turn on MMU, caches, etc.
	nop
	mov	pc, r2				@ jump to virtual addr
	nop
	nop
	nop
	nop
	
resume_vector_addr:
	.word	CONFIG_RESUME_VECTOR
	
resume_vector_backup:
	.word	0
	
sleep_save_sp:
	.word	0				@ preserve stack phys ptr here

resume_after_mmu:
	mov	r0, #0
	ldmfd	sp!, {r4 - r12, pc}		@ return to caller
