/*
 * arch/arm/mach-cxd4115/clock.c
 *
 * clock manager
 *
 * Copyright 2007,2008,2009 Sony Corporation
 *
 * This code is based on arch/arm/mach-integrator/clock.c.
 */
/*
 *  linux/arch/arm/mach-integrator/clock.c
 *
 *  Copyright (C) 2004 ARM Limited.
 *  Written by Deep Blue Solutions Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/errno.h>
#include <linux/err.h>
#include <linux/string.h>
#include <linux/clk.h>
#include <linux/mutex.h>
#include <asm/semaphore.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/arch/timex.h>
#include <asm/arch/regs-clk.h>

#include "clock.h"

/* 
 *  Enable/Disable UART BAUD clock.
 *    (note) UART PCLK should be always enabled.
 */
static int cxd4115_enable_uart(struct clk *clk, int enable)
{
	writel(enable, VA_CLKBLK_UART_CLK(clk->chan));
	return 0;
}

static struct clk clocks[] = {
	/*-------------------- UART --------------------*/
	{
		.name		= "UARTCLK0",
		.rate		= CXD4115_UART_CLK,
		.chan		= 0,
		.enable		= cxd4115_enable_uart,
	},
#ifdef CONFIG_CXD4115_ENABLE_UART1
	{
		.name		= "UARTCLK1",
		.rate		= CXD4115_UART_CLK,
		.chan		= 1,
		.enable		= cxd4115_enable_uart,
	},
#endif /* CONFIG_CXD4115_ENABLE_UART1 */
#ifdef CONFIG_CXD4115_ENABLE_UART2
	{
		.name		= "UARTCLK2",
		.rate		= CXD4115_UART_CLK,
		.chan		= 2,
		.enable		= cxd4115_enable_uart,
	},
#endif /* CONFIG_CXD4115_ENABLE_UART2 */
#ifdef CONFIG_ARCH_CXD4120
	{
		.name		= "UARTCLK3",
		.rate		= CXD4115_UART_CLK,
		.chan		= 3,
		.enable		= cxd4115_enable_uart,
	},
#endif /* CONFIG_ARCH_CXD4120 */
	/*----------------------------------------------*/
};

static DEFINE_MUTEX(clocks_mutex);

struct clk *clk_get(struct device *dev, const char *id)
{
	int i;
	struct clk *p, *clk = ERR_PTR(-ENOENT);

	mutex_lock(&clocks_mutex);
	for (i = 0, p = clocks; i < ARRAY_SIZE(clocks); i++, p++){
		if (strcmp(p->name, id) == 0){
			clk = p;
			break;
		}
	}
	mutex_unlock(&clocks_mutex);

	return clk;
}
EXPORT_SYMBOL(clk_get);

void clk_put(struct clk *clk)
{
}
EXPORT_SYMBOL(clk_put);

int clk_enable(struct clk *clk)
{
	mutex_lock(&clocks_mutex);
	(clk->enable)(clk, 1);
	mutex_unlock(&clocks_mutex);
	return 0;
}
EXPORT_SYMBOL(clk_enable);

void clk_disable(struct clk *clk)
{
	mutex_lock(&clocks_mutex);
	(clk->enable)(clk, 0);
	mutex_unlock(&clocks_mutex);
	return;
}
EXPORT_SYMBOL(clk_disable);

unsigned long clk_get_rate(struct clk *clk)
{
	if (clk->rate != 0)
		return clk->rate;
	if (clk->get_rate != NULL)
		return (clk->get_rate)(clk);
	return clk->rate;
}
EXPORT_SYMBOL(clk_get_rate);

long clk_round_rate(struct clk *clk, unsigned long rate)
{
	return rate;
}
EXPORT_SYMBOL(clk_round_rate);

int clk_set_rate(struct clk *clk, unsigned long rate)
{
	return 0;
}
EXPORT_SYMBOL(clk_set_rate);

static int __init clk_init(void)
{
	return 0;
}
arch_initcall(clk_init);
