/*
 * arch/arm/mach-cxd4115/core.c
 *
 * machine_desc and initialize function
 *
 * Copyright 2007,2008,2009 Sony Corporation
 *
 * This code is based on arch/arm/mach-realview/realview_eb.c.
 */
/*
 *  linux/arch/arm/mach-realview/realview_eb.c
 *
 *  Copyright (C) 2004 ARM Limited
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/device.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/sysdev.h>
#include <linux/mman.h>

#ifdef CONFIG_PM
#include <asm/arch/pm.h>
#endif /* CONFIG_PM */

#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/mach/map.h>
#include <asm/hardware.h>

#include <asm/io.h>
#include <asm/irq.h>

#include <asm/arch/gic.h>
#ifdef CONFIG_LOCAL_TIMER_TICK
# include <asm/hardware/arm_twd.h>
#else
# include <asm/arch/regs-timer.h>
#endif /* CONFIG_LOCAL_TIMER_TICK */
#include <asm/arch/regs-dmac.h>
#include <asm/arch/regs-emc.h>

/*---------------------------- I/O mapping --------------------------*/
/*
  Logical    Physical
  F8000000   78000000 Peripherals
  FE000000   FFF00000 Peripherals
*/
 
static struct map_desc cxd4115_io_desc[] __initdata = {
	{
		.virtual	= IO_ADDRESS(CXD4115_AHB_BASE),
		.pfn		= __phys_to_pfn(CXD4115_AHB_BASE),
		.length		= CXD4115_AHB_SIZE,
		.type		= MT_DEVICE },
	{
		.virtual	= IO_ADDRESS(CXD4115_APB_BASE),
		.pfn		= __phys_to_pfn(CXD4115_APB_BASE),
		.length		= CXD4115_APB_SIZE,
		.type		= MT_DEVICE },
	{
		.virtual	= HIO_ADDRESS(CXD4115_HIO_BASE),
		.pfn		= __phys_to_pfn(CXD4115_HIO_BASE),
		.length		= CXD4115_HIO_SIZE,
		.type		= MT_DEVICE },
};

void __init cxd4115_map_io(void)
{
	iotable_init(cxd4115_io_desc, ARRAY_SIZE(cxd4115_io_desc));
}

/*------------------------------ SYSDEV -----------------------------*/
#ifdef CONFIG_PM
static struct {
	int valid;
	u32 emc_bind;
	u32 emc_am2_cfg;
	u32 emc_am2_prm1;
	u32 emc_am2_prm2;
} sv = { 
	.valid = 0,
};

static int cxd4115_suspend(struct sys_device *dev, pm_message_t state)
{
	gic_suspend();

	/* Save EMC CS2 */
	sv.emc_bind = readl(VA_EMC + EMC_CS_PHY_BIND);
	sv.emc_am2_cfg = readl(VA_EMC + EMC_PHY_AM2_CFG);
	sv.emc_am2_prm1 = readl(VA_EMC + EMC_PHY_AM2_PRM1);
	sv.emc_am2_prm2 = readl(VA_EMC + EMC_PHY_AM2_PRM2);

	sv.valid = 1;
	return 0;
}

static int cxd4115_resume(struct sys_device *dev)
{
	/* Enable DMAC */
	writel(0x1, DMACCONFIGRATION);

	if (sv.valid) {
		/* Restore EMC CS2 */
		writel(sv.emc_bind, VA_EMC + EMC_CS_PHY_BIND);
		writel(sv.emc_am2_cfg, VA_EMC + EMC_PHY_AM2_CFG);
		writel(sv.emc_am2_prm1, VA_EMC + EMC_PHY_AM2_PRM1);
		writel(sv.emc_am2_prm2, VA_EMC + EMC_PHY_AM2_PRM2);

		sv.valid = 0;
	}

	gic_resume();
	return 0;
}
#endif /* CONFIG_PM */

static struct sysdev_class cxd4115_sysclass = {
	set_kset_name("cxd4115"),
#ifdef CONFIG_PM
	.suspend	= cxd4115_suspend,
	.resume		= cxd4115_resume,
#endif
};

static struct sys_device cxd4115_device = {
	.id		= 0,
	.cls		= &cxd4115_sysclass,
};

int __init cxd4115_core_init(void)
{
	int ret;
#ifdef CONFIG_OVERCOMMIT_ALWAYS
	extern int sysctl_overcommit_memory;
#endif

#ifdef CONFIG_PM
	pm_power_off = cxd4115_poweroff;
#endif
#ifdef CONFIG_OVERCOMMIT_ALWAYS
	sysctl_overcommit_memory = OVERCOMMIT_ALWAYS;
#endif

	/* 
	 *  Enable DMAC0
	 *    (note) DMAC0 HCLK is enabled (default).
	 */
	writel(0x1, DMACCONFIGRATION);

	ret = sysdev_class_register(&cxd4115_sysclass);
	if (ret) {
		printk(KERN_ERR "cxd4115_core_init: sysdev_class_register failed.\n");
		return ret;
	}
	ret = sysdev_register(&cxd4115_device);
	if (ret) {
		printk(KERN_ERR "cxd4115_core_init: sysdev_register failed.\n");
		return ret;
	}
	return ret;
}

void __init cxd4115_init_irq(void)
{
	gic_dist_init(0, VA_GIC_DIST, 29);
	gic_cpu_init(0, VA_GIC_CPU);
}

#ifdef CONFIG_MEMORY_HOTPLUG
unsigned int target_get_mem_type(unsigned long phys)
{
	return MT_HOTPLUG;
}
#endif
