/*
 * arch/arm/mach-cxd4115/pm.c
 *
 * pm callback for cxd4115
 *
 * Copyright 2007,2008,2009 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/pm.c.
 */
/*
 * SA1100 Power Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06:	Cliff Brake         Initial code
 *
 * 2001-02-25:	Sukjae Cho <sjcho@east.isi.edu> &
 * 		Chester Kuo <chester@linux.org.tw>
 * 			Save more value for the resume function! Support
 * 			Bitsy/Assabet/Freebird board
 *
 * 2001-08-29:	Nicolas Pitre <nico@cam.org>
 * 			Cleaned up, pushed platform dependent stuff
 * 			in the platform specific files.
 *
 * 2002-05-27:	Nicolas Pitre	Killed sleep.h and the kmalloced save array.
 * 				Storage is local on the stack now.
 */
#include <linux/init.h>
#include <linux/suspend.h>
#include <linux/time.h>
#include <linux/bootmem.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/page.h>
#include <asm/mach/arch.h>
#include <asm/arch/system.h>
#include <asm/mach/time.h>
#include <asm/arch/time.h>
#include <asm/arch/pm.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/mach/irq.h>
#include <linux/delay.h>
#include <asm/arch/regs-dmac.h>
#include <asm/arch/regs-clk.h>
#include <asm/arch/regs-gpio.h>
#include <asm/arch/regs-misc.h>
#include <asm/arch/regs-timer.h>
#include <asm/arch/hardware.h>
#include <asm/arch/memory.h>

#include <asm/mach/warmboot.h>

extern int cxd4115_cpu_suspend(int);
extern void cxd4115_cpu_resume(void);

static int _cxd4115_pm_enter(int mode)
{
	int err=0;
#ifdef CONFIG_SNSC_DEBUG_PROFILE_INITIAL_SUSPEND
	extern void profile_soft_init(void);
#endif
	
	printk("resume vector: %08x\n", CONFIG_RESUME_VECTOR);

	/* go zzz */
	if(cxd4115_cpu_suspend(mode)) {
		err = -1;
		printk("suspend error!\n");
	}

	cpu_init();

#ifdef CONFIG_SNSC_DEBUG_PROFILE_INITIAL_SUSPEND
	profile_soft_init();
#endif
	cxd4115_timer_early_init();
	
	return err;
}

unsigned long sleep_phys_sp(void *sp)
{
	return virt_to_phys(sp);
}

static u8 reboot_reason;
static u16 reboot_timer;
static u16 state_timer;

static void cxd4115_pm_halt(int reboot)
{
	cxd4115_cpu_suspend((reboot)?PM_ENTER_RESET:PM_ENTER_ERROR);

	/* never reached */
}

void pm_suspend_error(int mode)
{
	cxd4115_pm_halt(0);
	/* never reached */
}
		
/*
 * Called after processes are frozen, but before we shut down devices.
 */
static int cxd4115_pm_prepare(suspend_state_t state)
{
	return warm_boot_pm_prepare(state);
}

static int cxd4115_pm_enter(suspend_state_t state)
{
	return _cxd4115_pm_enter(PM_ENTER_NORMAL);
}

/*
 * Called after devices are re-setup, but before processes are thawed.
 */
static int cxd4115_pm_finish(suspend_state_t state)
{
	reboot_reason = 0;
	reboot_timer = 0;
	state_timer = 0;

	return warm_boot_pm_finish(state);
}

static int cxd4115_valid(suspend_state_t state)
{
	return state == PM_SUSPEND_MEM || state == PM_SUSPEND_DISK;
}

/*
 * Set to PM_DISK_FIRMWARE so we can quickly veto suspend-to-disk.
 */
static struct pm_ops cxd4115_pm_ops = {
	.prepare	= cxd4115_pm_prepare,
	.enter		= cxd4115_pm_enter,
	.finish		= cxd4115_pm_finish,
	.valid		= cxd4115_valid,
};

extern struct kset power_subsys;

#define wb_attr(_name) \
static ssize_t _name##_show(struct kset * s, char * buf)            \
{                                                                        \
	return sprintf(buf, "%u\n", _name);                              \
}                                                                        \
static ssize_t _name##_store(struct kset * s,                       \
			     const char * buf, size_t n)                 \
{                                                                        \
	char *endp;                                                      \
	int value;                                                       \
	                                                                 \
	value = simple_strtoul(buf, &endp, 0);                           \
	if(*endp)                                                        \
		return -EINVAL;                                          \
                                                                         \
	_name = value;                                                   \
	return n;                                                        \
}                                                                        \
static struct subsys_attribute _name##_attr = {	                         \
	.attr	= {				                         \
		.name = __stringify(_name),	                         \
		.mode = 0644,			                         \
	},					                         \
	.show	= _name##_show,			                         \
	.store	= _name##_store,		                         \
}

wb_attr(reboot_reason);
wb_attr(reboot_timer);
wb_attr(state_timer);

static int __init cxd4115_pm_init(void)
{
	int error = 0;

	error = subsys_create_file(&power_subsys, &reboot_reason_attr);
	if (error) {
		goto out;
	}
	error = subsys_create_file(&power_subsys, &reboot_timer_attr);
	if (error) {
		goto out;
	}
	error = subsys_create_file(&power_subsys, &state_timer_attr);
	if (error) {
		goto out;
	}
	
	pm_set_ops(&cxd4115_pm_ops);
 out:
	return error;
}

late_initcall(cxd4115_pm_init);

#ifdef CONFIG_WARM_BOOT_IMAGE
int cxd4115_create_warmbootimage(void)
{
	return cxd_create_warmbootimage((unsigned long)cxd4115_cpu_resume, 0);
}
#else
int cxd4115_create_warmbootimage(void)
{
	return 0;
}
#endif

static void cxd4115_reset(void)
{
	local_irq_disable();
	/* assert XRESET_REQ */
	writel(1, VA_MISCCTRL(MISCCTRL_XRESETREQ)+MISCCTRL_CLR);

	while (1) ;
}
void (*pm_machine_reset)(void) = cxd4115_reset;
EXPORT_SYMBOL(pm_machine_reset);

void cxd4115_poweroff(void)
{
	local_irq_disable();

#ifdef CONFIG_CXD4115_POWEROFF_SUSPEND
	cxd4115_pm_halt(0);
#else
	/* DATA=L */
	writel(XPOWER_OFF, VA_GPIO(GPIO_POWEROFF)+GPIO_DATA_CLEAR);
	/* DIR=Output */
	gpio_set(GPIO_POWEROFF, GPIO_DIR, XPOWER_OFF_BIT);
#endif

	while (1) ;
}
