/*
 * arch/arm/mach-cxd4115/sleep.S
 *
 * pm callback for cxd4115
 *
 * Copyright 2007,2008 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/sleep.S
 */
/*
 * SA11x0 Assembler Sleep/WakeUp Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06: Cliff Brake         Initial code
 *
 * 2001-08-29:	Nicolas Pitre	Simplified.
 *
 * 2002-05-27:	Nicolas Pitre	Revisited, more cleanup and simplification.
 *				Storage is on the stack now.
 */

#include <linux/linkage.h>
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <asm/arch/regs-gpio.h>
#include <asm/arch/regs-misc.h>
#include <asm/arch/regs-emc.h>
#include <asm/arch/regs-dmc.h>
#include <asm/arch/regs-prc.h>
#include <asm/arch/regs-clk.h>	
#include <asm/arch/regs-bam.h>
#include <asm/arch/debug-macro.S>
#include <asm/arch/bootram.h>

#include <asm/arch/pm.h>

#define RESUMESRAM_BASE (VA_BOOTRAM + CXD4115_SUSPEND_WORKAREA)
#define RESUMESRAM_PHYS (CXD4115_BOOTRAM_BASE + CXD4115_SUSPEND_WORKAREA)

/* below is used in system MMU off */
/*
	SDRAM control region base
*/
#define EMC_BASE CXD4115_EMC_BASE
/*
	DDR contorl region base
*/
#define SDC_BASE CXD4115_SDC_BASE

#ifdef CONFIG_ARCH_CXD4120
# define SDC1_BASE CXD4115_SDC(1)
#endif /* CONFIG_ARCH_CXD4120 */

#define SHARED_MEM	CONFIG_SUSPEND_STATUS_ADDR
#define		ERROR	0
#define		STATUS  1
#define		SUSCMD	2
#define		PWRKEY  3

		.data


/*
 * cxd4115_cpu_suspend()
 *
 * Causes cxd4115 to enter sleep state
 *
 */

ENTRY(cxd4115_cpu_suspend)
	@ r0: mode flag

	stmfd	sp!, {r4 - r12, lr}		@ save registers on stack
	str	r0, modeflag			@ save mode flag

	@ get coprocessor registers
	mrc 	p15, 0, r4, c3, c0, 0		@ domain ID
	mrc 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mrc	p15, 0, r6, c13, c0, 0		@ PID
	mrc	p15, 0, r7, c13, c0, 1		@ context ID
	mrc	p15, 0, r8, c13, c0, 3		@ TLS
	mrc 	p15, 0, r9, c1, c0, 0		@ control reg
	mrc 	p15, 0, r10, c1, c0, 1		@ aux control reg
	
	@ store them plus current virtual stack ptr on stack
	mov	r11, sp
	stmfd	sp!, {r4 - r11}

	@ preserve phys address of stack
	mov	r0, sp
	bl	sleep_phys_sp
	ldr	r1, =sleep_save_sp
	str	r0, [r1]

	@ disable interrupt
	mov	r0, #PSR_F_BIT | PSR_I_BIT | SVC_MODE
	msr	cpsr_c, r0

	@ clean & invalide D&I cache, necessary?
	bl	v6_flush_kern_cache_all
	
	ldr	r0, modeflag
	cmp	r0, #PM_ENTER_RESET
	beq	goto_suspend

	@ for creating Warm-Boot-Image
	bl	cxd4115_create_warmbootimage

	cmp	r0, #0
	beq	goto_suspend

	/*
		failure handling for warm_boot_image
	*/
	mov	r0, sp
	ldmfd	r0, {r4 - r10, sp}
	mcr 	p15, 0, r4, c3, c0, 0		@ domain ID
	mcr 	p15, 0, r5, c2, c0, 0		@ translation table base addr
	mcr	p15, 0, r6, c13, c0, 0		@ PID
	mrc	p15, 0, r7, c13, c0, 1		@ context ID
	mrc	p15, 0, r8, c13, c0, 3		@ TLS
	mrc 	p15, 0, r9, c1, c0, 0		@ control reg
	mrc 	p15, 0, r10, c1, c0, 1		@ aux control reg
	ldmfd	sp!, {r4 - r12, pc}

goto_suspend:
#ifdef CONFIG_SNSC_DEBUG_PROFILE
	adr	r0, __printascii
	bl	profile_dump_record
#endif
	@ copy the standby operation code to the resume ram
	ldr	r0, =RESUMESRAM_BASE
	adr	r1, exec_in_resume_ram
	adr	r2, exec_in_resume_ram_end

code_copy:		
	ldr	r3, [r1], #4
	str	r3, [r0], #4
	teq	r1, r2
	bne	code_copy

	@ r4-r11 are for control region base address
	@ load neccesary data to register to avoid accessing sdr/ddr
	ldr	r4, =SDC_BASE
	ldr	r5, =EMC_BASE
	ldr	r6, =SHARED_MEM
	ldr	r7, resume_vector_addr		@ address of the end of DDR	
	@ physical address of cxd4115_cpu_resume
	ldr	r0, =cxd4115_cpu_resume
	bl	sleep_phys_sp
	mov	r8, r0				@ resume vector
	bl	__printhex8
	mov	r0, #':'
	bl	__printch
	@ resume ram base address
	ldr	r9, =RESUMESRAM_PHYS		@ Physical address
	@ resume_vector_backup phys addr
	ldr	r0, =resume_vector_backup
	bl	sleep_phys_sp
	mov	r10, r0				
	@ loops per jiffy
	ldr	r0, LC0				
	ldr	r11, [r0]
	
	@ clean & invalide D&I cache
	bl	v6_flush_kern_cache_all
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x1000			@ ...i............
	bic	r0, r0, #0x0006			@ .............ca.
	mcr	p15, 0, r0, c1, c0, 0		@ disable caches
 	@ disable MMU, the mapping is flat so
 	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
 	bic	r0, r0, #0x0001			@ ...............m
 	mcr	p15, 0, r0, c1, c0, 0		@ disable MMU
	@ jump to the resume ram(Physical address)
	mov	pc, r9				@ fetch: MMU active
	nop					@ fetch: MMU active
	nop

	.ltorg
	.align	5
exec_in_resume_ram:
	@ Invert STABLE_RUN for DEBUG
	ldr	r0, =CXD4115_GPIO(GPIO_STABLERUN)
	ldr	r1, [r0, #GPIO_DATA_READ]
	ands	r1, r1, #STABLE_RUN
	mov	r1, #STABLE_RUN
	streq	r1, [r0, #GPIO_DATA_SET]
	strne	r1, [r0, #GPIO_DATA_CLEAR]

	@ save resume vector
	ldr	r0, [r7]			@ load original data
	str	r0, [r10]			@ save to resume_vector_backup
	str	r8, [r7]			@ save resume vector

#ifndef CONFIG_ARCH_CXD4120
	@ send message
	mov	r0, #0
	strb	r0, [r6, #ERROR]
	strb	r0, [r6, #STATUS]
	strb	r0, [r6, #SUSCMD]

	@ force RESET ?
	ldr	r0, modeflag
	cmp	r0, #PM_ENTER_RESET
	beq	chk_err

	mov	r0, #'\n'
	bl	__printch
	ldr	r0, [r6]
	bl	__printhex8
	mov	r0, #'-'
	bl	__printch

	mov	r0, #1
	strb	r0, [r6, #SUSCMD]
retry:	ldrb	r0, [r6, #STATUS]
	cmp	r0, #0
	beq	retry

	ldr	r0, [r6]
	bl	__printhex8
	mov	r0, #'\n'
	bl	__printch

chk_err:
	ldr	r1, modeflag
	cmp	r1, #PM_ENTER_NORMAL
	@ if !PM_ENTER_NORMAL, set ERROR flag for Loader
	movne	r0, #1
	strneb	r0, [r6, #ERROR]
	cmp	r1, #PM_ENTER_RESET
	beq	set_reboot

	ldrb	r0, [r6, #PWRKEY]		@ Power Key Status
	cmp	r0, #0
	beq	set_reboot_end
set_reboot:
	mov	r0, #1
	str	r0, reboot
set_reboot_end:
#endif /* CONFIG_ARCH_CXD4120 */

#ifdef CONFIG_CXD4115_DDR_ACCESS_DETECT
	ldr	r1, =CXD4115_BAM_BASE
	@ Monitor DMC0 from ALL IP
	mov	r0, #BAM_AXI_MON1_DMC0_ALL	@ DMC0, IP59-IP32
	str	r0, [r1, #BAM_AXI_MON1_IP1]
	mov	r0, #BAM_AXI_MON1_IP2_ALL	@ IP31-IP0
	str	r0, [r1, #BAM_AXI_MON1_IP2]
	@ start
	mov	r0, #BAM_MODE_COUNT|BAM_CTRL_CPU|BAM_START
	str	r0, [r1, #BAM_LOAD_SET]
	@ wait 20ms
	ldr	r0, =20000
	bl	__udelay			@ destruct r0, r2
	@ stop
	mov	r0, #BAM_MODE_COUNT|BAM_CTRL_CPU
	str	r0, [r1, #BAM_LOAD_SET]
	@ show result
	ldr	r0, [r1, #BAM_MON1_BURST_COUNT]
	bl	__printhex8			@ destruct r0-r3
	mov	r0, #'\n'
	bl	__printch
#endif /* CONFIG_CXD4115_DDR_ACCESS_DETECT */
	/*
	 * DDR power down
	 */
#ifdef CONFIG_DDR01_CONTROL
	@ set DDRs to SREF or DPD
	mov	r1, #0				@ SREF command word
	mov	r2, #0				@ DPD  command word
	ldr	r0, [r4, #DMC_MEMORY]
	ands	r0, r0, #DMC_MODE2RANK		@ 2RANKs populated ?
#ifdef CONFIG_CXD4115_DDR1_DPD
	orrne	r2, r2, #DMC_APBCMDREQ1		@ set DDR1 to DPD, if exists.
#else
	orrne	r1, r1, #DMC_APBCMDREQ1		@ set DDR1 to SREF, if exists.
#endif
#ifdef CONFIG_CXD4115_DDR0_DPD
	orr	r2, r2, #DMC_APBCMDREQ0		@ set DDR0 to DPD
#else
	orr	r1, r1, #DMC_APBCMDREQ0		@ set DDR0 to SREF
#endif
	bl	__dmc_issue
#endif /* CONFIG_DDR01_CONTROL */

	/*
	 * DDR1 power down
	 */
#if defined CONFIG_ARCH_CXD4120 && defined CONFIG_DDR23_CONTROL
	ldr	r4, =SDC1_BASE
	@ set DDRs to SREF or DPD
	mov	r1, #0				@ SREF command word
	mov	r2, #0				@ DPD  command word
	ldr	r0, [r4, #DMC_MEMORY]
	ands	r0, r0, #DMC_MODE2RANK		@ 2RANKs populated ?
#ifdef CONFIG_CXD4115_DDR3_DPD
	orrne	r2, r2, #DMC_APBCMDREQ1		@ set DDR3 to DPD, if exists.
#else
	orrne	r1, r1, #DMC_APBCMDREQ1		@ set DDR3 to SREF, if exists.
#endif
#ifdef CONFIG_CXD4115_DDR2_DPD
	orr	r2, r2, #DMC_APBCMDREQ0		@ set DDR2 to DPD
#else
	orr	r1, r1, #DMC_APBCMDREQ0		@ set DDR2 to SREF
#endif
	bl	__dmc_issue
#endif /* CONFIG_DDR23_CONTROL && CONFIG_ARCH_CXD4120 */

	/*
	 * DMC reset
	 */
#ifndef CONFIG_CXD4115_DDRPHY_RESET
	/* By TYPEID */
	ldr	r4, =CXD4115_MISCCTRL(MISCCTRL_TYPEID)
	ldr	r0, [r4]
	and	r0, r0, #TYPEID_IDMASK
	cmp	r0, #TYPEID_CXD4120
	beq	1f
	cmp	r0, #TYPEID_CXD9938
	beq	1f
#endif /* !CONFIG_CXD4115_DDRPHY_RESET */
	@ wait 10us (Undocumented. not mandatory.)
	mov	r0, #10
	bl	__udelay

	ldr	r4, =CXD4115_PRC_BASE
	ldr	r0, [r4, #PRC_PHY_MODE1_MST]
	ldr	r1, =PRC_PHY_RESET_MASK
	and	r0, r0, r1
	str	r0, [r4, #PRC_PHY_MODE1_MST]	@ reset
1:

	/*
	 * SDR power down
	 */
#ifdef CONFIG_CXD4115_USE_SDR
	@ set SDRAM to SREF or DPD
# ifdef CONFIG_CXD4115_EMC_DETECT
	mov	r0, #0
	str	r0, [r5, #EMC_BM_CTRL]
	ldr	r0, =0x80280000			@ period 16.7ms
	str	r0, [r5, #EMC_CNT_MODE]
	mov	r0, #EMC_BP_ON
	str	r0, [r5, #EMC_BP_ON_MODE]	@ count start
# endif /* CONFIG_CXD4115_EMC_DETECT */
	@ Flush FIFOs
	mov	r0, #0x07
	str	r0, [r5, #EMC_TRANS_CTRL]
1:
	ldr	r0, [r5, #EMC_TRANS_CTRL]
	cmp	r0, #0
	bne	1b

        mov	r0, #0x8000
# ifdef CONFIG_CXD4115_SDR_DPD
	orr	r0, r0, #EMC_SDR_SEQCMD_DPD	@ DPD
# else
	orr	r0, r0, #EMC_SDR_SEQCMD_SREF	@ SREF
# endif
        str	r0, [r5, #EMC_PHY_SDR_CMD]	@ issue
	@ delay > 1ms for SDR SREF
	mov	r0, #0x400
	bl	__udelay
#endif /* CONFIG_CXD4115_USE_SDR */

	mov	r0, #'E'
	bl	__printch
	mov	r0, #'\r'
	bl	__printch
	mov	r0, #'\n'
	bl	__printch

#ifdef CONFIG_CXD4115_EMC_DETECT
	ldr	r0, =20000			@ wait 20ms
	bl	__udelay
	@ show result
	add	r8, r5, #EMC_C0_SINGLE_WR
	mov	r7, #0x17
1:
	ldr	r0, [r8], #4
	bl	__printhex8
	ands	r0, r7, #0x7
	moveq	r0, #'\n'
	movne	r0, #' '
	bl	__printch
	subs	r7, r7, #1
	bpl	1b
#endif /* CONFIG_CXD4115_EMC_DETECT */

	/*
	 * D-D Conv.  control
	 */
#ifndef CONFIG_ARCH_CXD4120
	ldr	r0, reboot
	cmp	r0, #1
	beq	dd_reboot
#endif /* CONFIG_ARCH_CXD4120 */
dd_off:
	ldr	r4, =CXD4115_GPIO(GPIO_POWEROFF)
	mov	r5, #XPOWER_OFF
	str	r5, [r4, #GPIO_DATA_CLEAR]	@ assert
	b	end_job
#ifndef CONFIG_ARCH_CXD4120
dd_reboot:
	ldr	r4, =CXD4115_MISCCTRL(MISCCTRL_XRESETREQ)
	mov	r5, #1
	str	r5, [r4, #MISCCTRL_CLR]		@ assert
#endif /* CONFIG_ARCH_CXD4120 */
end_job:
	@ wait for interrupt
	mov	r0, #0
1:
	mcr	p15, 0, r0, c7, c0, 4
	b	1b

/* 
 * issue DMC_COMMAND
 *    input:	r4=DMC base address, r1=SREF param, r2=DPD param
 *    output:	none.
 *    destruct: r0, r3
 */
__dmc_issue:
	mov	r3, lr				@ save lr
	bl	__dmc_ready			@ wait for ready
	cmp	r1, #0
	beq	1f
	orr	r1, r1, #DMC_APBCMD_SIN		@ SREF Enter command
	str	r1, [r4, #DMC_COMMAND]		@ issue
	bl	__dmc_ready			@ wait for done
1:
	cmp	r2, #0
	beq	1f
	mov	r0, #0
	str	r0, [r4, #DMC_REFRESH]		@ stop auto-refresh
	orr	r2, r2, #DMC_APBCMD_DPD		@ DPD command
	str	r2, [r4, #DMC_COMMAND]		@ issue
	bl	__dmc_ready			@ wait for done
1:
	mov	pc, r3

/* 
 * Wait for DMC_COMMAND register ready
 *    input:	r4=DMC base address
 *    output:	none.
 *    destruct:	r0
 */
__dmc_ready:
	ldr	r0, [r4, #DMC_COMMAND]
	ands	r0, r0, #DMC_APBCMDREQMASK
	bne	__dmc_ready
	mov	pc, lr

LC0:		.word	loops_per_jiffy

/* copy from arch/arm/lib/delay.S
 * 0 <= r0 <= 2000
 */
__udelay:
		ldr	r2, =0x68db
		mul	r0, r2, r0

		mov	r2, r11
		mov	r0, r0, lsr #11		@ max = 0x00003fff
		mov	r2, r2, lsr #11		@ max = 0x0003ffff
		mul	r0, r2, r0		@ max = 2^32-1
		movs	r0, r0, lsr #6
		moveq	pc,lr
1:		subs	r0, r0, #1
		bhi	1b
		mov	pc,lr

modeflag:	.long	0x00
#ifndef CONFIG_ARCH_CXD4120
reboot:		.long	0x00
#endif /* CONFIG_ARCH_CXD4120 */

#if defined(CONFIG_DEBUG_LL)
	@ for debug
__printhex8:	
	mov	r1, #8
	b	__printhex

__printhex4:	
	mov	r1, #4
	b	__printhex

__printhex2:	
	mov	r1, #2
__printhex:
	adr	r2, hexbuf
	add	r3, r2, r1
	mov	r1, #0
	strb	r1, [r3]
1:	and	r1, r0, #15
	mov	r0, r0, lsr #4
	cmp	r1, #10
	addlt	r1, r1, #'0'
	addge	r1, r1, #'a' - 10
	strb	r1, [r3, #-1]!
	teq	r3, r2
	bne	1b
	mov	r0, r2
	b	__printascii

__printascii:	
	addruart r3
	b	2f
1:	waituart r2, r3
	senduart r1, r3
	busyuart r2, r3
	teq	r1, #'\n'
	moveq	r1, #'\r'
	beq	1b
2:	teq	r0, #0
	ldrneb	r1, [r0], #1
	teqne	r1, #0
	bne	1b
	mov	pc, lr

__printch:	
	addruart r3
	mov	r1, r0
	mov	r0, #0
	b	1b

hexbuf:		.space 16
#else
__printhex8:
	mov	pc, lr
__printhex4:
	mov	pc, lr
__printhex2:
	mov	pc, lr
__printhex:
	mov	pc, lr
__printascii:
	mov	pc, lr
__printch:
	mov	pc, lr
#endif
	.ltorg
exec_in_resume_ram_end:

/*
 * cxd4115_cpu_resume()
 *
 * entry point from bootloader into kernel during resume
 *   Redboot does not have resume vector.
 *   Please jump to this vector by "go" command of RedBoot.
 */
/* 
 * Execute only 1 CPU.
 * Because, other CPUs are offline before sleep.
 */
	.align 5
ENTRY(cxd4115_cpu_resume)
	mov	r0, #PSR_F_BIT | PSR_I_BIT | SVC_MODE
	msr	cpsr_c, r0			@ set SVC, irqs off

	mov	r0, #'R'
	bl	__printch

	mov	r0, #0
	mcr	p15, 0, r0, c7, c14, 0		@ clean+invalidate D cache
	mcr	p15, 0, r0, c7, c5, 0		@ invalidate I+BTB cache
	mcr	p15, 0, r0, c7, c10, 4		@ drain write buffer
	mcr	p15, 0, r0, c8, c7, 0		@ invalidate I + D TLBs
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x0007			@ .............cam
	bic	r0, r0, #0x1100			@ ...i...s........
	mcr	p15, 0, r0, c1, c0, 0		@ ctrl register
	nop
	nop

	@ restore data of resume vector
	ldr	r0, resume_vector_backup
	ldr	r1, resume_vector_addr
	str	r0, [r1]
 
	ldr	r0, sleep_save_sp		@ stack phys addr
	ldr	r2, =resume_after_mmu		@ its absolute virtual address
	ldmfd	r0, {r4 - r10, sp}		@ CP regs + virt stack ptr

	mov	r0, #0
	mcr 	p15, 0, r4, c3, c0, 0		@ domain ID
	mcr	p15, 0, r0, c2, c0, 2		@ TTB control register
	mcr 	p15, 0, r5, c2, c0, 0		@ TTB0
	mcr	p15, 0, r0, c2, c0, 1		@ TTB1
	mcr	p15, 0, r6, c13, c0, 0		@ PID
	mcr	p15, 0, r7, c13, c0, 1		@ context ID
	mcr	p15, 0, r8, c13, c0, 3		@ TLS
#ifdef CONFIG_SMP
	/* Set up the SCU */
	ldr	r0, =SCU_BASE
	ldr	r5, [r0, #SCU_CTRL]
	orr	r5, r5, #1
	str	r5, [r0, #SCU_CTRL]
# ifndef CONFIG_CPU_DCACHE_DISABLE
	orr	r10, r10, #0x20			@ Enable SMP/nAMP mode
# endif
#endif /* CONFIG_SMP */
	mcr 	p15, 0, r10, c1, c0, 1		@ aux control reg.
	b	resume_turn_on_mmu		@ cache align execution

	.align 5
resume_turn_on_mmu:
	mcr 	p15, 0, r9, c1, c0, 0		@ turn on MMU, caches, etc.
	nop
	mov	pc, r2				@ jump to virtual addr
	nop
	nop
	nop
	nop
	
resume_vector_addr:
	.word	CONFIG_RESUME_VECTOR
	
resume_vector_backup:
	.word	0
	
sleep_save_sp:
	.word	0				@ preserve stack phys ptr here

resume_after_mmu:
	mov	r0, #0
	ldmfd	sp!, {r4 - r12, pc}		@ return to caller
