/*
 * arch/arm/mach-cxd4115/time.c
 *
 * timer driver
 *
 * Copyright 2007,2008,2009 Sony Corporation
 *
 * This code is based on arch/arm/mach-integrator/core.c.
 */
/*
 *  linux/arch/arm/mach-integrator/core.c
 *
 *  Copyright (C) 2000-2003 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2, as
 * published by the Free Software Foundation.
 */
#include <linux/device.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/pm.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>

#include <asm/mach/time.h>
#include <asm/arch/time.h>
#ifdef CONFIG_LOCAL_TIMER_TICK
# include <asm/arch/localtimer.h>
# include <asm/hardware/arm_twd.h>
#else
# include <asm/arch/regs-timer.h>
#endif /* CONFIG_LOCAL_TIMER_TICK */
#include <asm/arch/gic.h>

/* sched clock */
unsigned long long sched_clock(void)
{
	unsigned long long v;

#ifdef CONFIG_LOCAL_TIMER_TICK
	v = (unsigned long long)(~readl(VA_LOCALTIMER + TWD_WDOG_COUNTER)); /* WDOG is down counter */
#else /* CONFIG_LOCAL_TIMER_TICK */
	v = (unsigned long long)readl(VA_TIMER(TIMER_FOR_SCHED)+CXD4105_TIMERREAD);
#endif /* CONFIG_LOCAL_TIMER_TICK */
	return v * (1000000000/SCHED_TICK_RATE);
}

unsigned long long read_sched_clock(void)
{
	unsigned long long v;

#ifdef CONFIG_LOCAL_TIMER_TICK
	v = (unsigned long long)(~readl(VA_LOCALTIMER + TWD_WDOG_COUNTER)); /* WDOG is down counter */
#else /* CONFIG_LOCAL_TIMER_TICK */
	v = (unsigned long long)readl(VA_TIMER(TIMER_FOR_SCHED)+CXD4105_TIMERREAD);
#endif /* CONFIG_LOCAL_TIMER_TICK */
	return v * (1000000000/SCHED_TICK_RATE);
}

/*
 * Returns number of ms since last clock interrupt.  Note that interrupts
 * will have been disabled by do_gettimeoffset()
 */
static unsigned long cxd4115_gettimeoffset(void)
{
	unsigned long ticks1;
#ifdef CONFIG_LOCAL_TIMER_TICK
	unsigned long ticks2;
#endif

	/* If TimerRead counter reaches TimerCompare value
	 * the counter will be cleared, so that just read TimerRead value.
	 */

#ifdef CONFIG_LOCAL_TIMER_TICK
	ticks2 = readl(VA_LOCALTIMER + TWD_TIMER_COUNTER);
	ticks1 = TIMER_RELOAD - ticks2; /* down counter. */
	if (readl(VA_LOCALTIMER + TWD_TIMER_INTSTAT) & 0x1) { /* pending ? */
		if (ticks2 > 0)
			ticks1 += TIMER_RELOAD;
	}
#else /* CONFIG_LOCAL_TIMER_TICK */
	ticks1 = readl(VA_TIMER(TIMER_FOR_TICK) + CXD4105_TIMERREAD);
	if (GIC_IRQ_PENDING(IRQ_TIMER(TIMER_FOR_TICK))) { /* pending ? */
		if (ticks1 < TIMER_RELOAD-1)
			ticks1 += TIMER_RELOAD;
	}
#endif /* CONFIG_LOCAL_TIMER_TICK */
	return TICKS2USECS(ticks1);
}

/*
 * IRQ handler for the timer
 */
extern irqreturn_t cxd4115_timer_interrupt(int, void *);

irqreturn_t cxd4115_timer_interrupt(int irq, void *dev_id)
{
	write_seqlock(&xtime_lock);

	// ...clear the interrupt
	// Counter automatically cleared in periodic mode.
#ifndef CONFIG_LOCAL_TIMER_TICK
	writel(1, VA_TIMER(TIMER_FOR_TICK)+CXD4105_TIMERCLR);
#endif /* !CONFIG_LOCAL_TIMER_TICK */

	timer_tick();

#if defined(CONFIG_SMP) && !defined(CONFIG_LOCAL_TIMERS)
        smp_send_timer();
        update_process_times(user_mode(get_irq_regs()));
#endif

	write_sequnlock(&xtime_lock);

	return IRQ_HANDLED;
}

#ifndef CONFIG_LOCAL_TIMER_TICK
static struct irqaction cxd4115_timer_irq = {
	.name		= "CXD4115 timer",
	.flags		= IRQF_DISABLED | IRQF_TIMER | IRQF_IRQPOLL,
	.handler	= cxd4115_timer_interrupt
};
#endif /* !CONFIG_LOCAL_TIMER_TICK */

/*
 * Setup sched clock(profile timer)
 * 	Called when resume (cxd4115_pm_enter()).
 */
void cxd4115_timer_early_init(void)
{
#ifdef CONFIG_LOCAL_TIMER_TICK
	local_wdog_setup();
#else /* CONFIG_LOCAL_TIMER_TICK */
	/* (note) TIMER CLK and PCLK are enabled. (default) */
	/* Clear the sched clock */
	writel(0, VA_TIMER(TIMER_FOR_SCHED)+CXD4105_TIMERCTL);
	writel(TMCLR|TMINTCLR, VA_TIMER(TIMER_FOR_SCHED)+CXD4105_TIMERCLR);
	/* Start the sched clock */
	writel(CXD4115_TMCK_SCHED_DIV|TMCS_FREERUN|TMST, VA_TIMER(TIMER_FOR_SCHED)+CXD4105_TIMERCTL);
#endif /* CONFIG_LOCAL_TIMER_TICK */
}

#ifndef CONFIG_LOCAL_TIMER_TICK
static inline void cxd4115_systimer_clear(void)
{
	/* Clear the system timer */
	writel(0, VA_TIMER(TIMER_FOR_TICK)+CXD4105_TIMERCTL);
	writel(TMCLR|TMINTCLR, VA_TIMER(TIMER_FOR_TICK)+CXD4105_TIMERCLR);
}

static inline void cxd4115_systimer_setup(void)
{
	/* Setup the system timer */
	writel(CXD4115_TMCK_TICK_DIV|TMCS_PERIODIC|TMINT, VA_TIMER(TIMER_FOR_TICK)+CXD4105_TIMERCTL);
	writel(TIMER_RELOAD, VA_TIMER(TIMER_FOR_TICK)+CXD4105_TIMERCMP);
}

static inline void cxd4115_systimer_start(void)
{
	/* Start the system timer */
	writel(CXD4115_TMCK_TICK_DIV|TMCS_PERIODIC|TMINT|TMST, VA_TIMER(TIMER_FOR_TICK)+CXD4105_TIMERCTL);
}
#endif /* !CONFIG_LOCAL_TIMER_TICK */

static void __init cxd4115_init_timer(void)
{
#ifdef CONFIG_LOCAL_TIMER_TICK
	local_timer_setup();
#else /* CONFIG_LOCAL_TIMER_TICK */
	/* (note) TIMER CLK and PCLK are enabled. (default) */
	cxd4115_systimer_clear();
	cxd4115_systimer_setup();
	setup_irq(IRQ_TIMER(TIMER_FOR_TICK), &cxd4115_timer_irq);
	cxd4115_systimer_start();
#endif /* CONFIG_LOCAL_TIMER_TICK */

#ifdef CONFIG_CXD4115_PRINTK_SCHED_CLOCK
	/* This should be done after map_io. (paging_init) */
	safe_to_call_sched_clock = 1;
#endif /* CONFIG_CXD4115_PRINTK_SCHED_CLOCK */
}

#ifdef CONFIG_PM
static void suspend_timer(void)
{
	/* Stop the system timer */
# ifdef CONFIG_LOCAL_TIMER_TICK
	local_timer_stop(0);
# else /* CONFIG_LOCAL_TIMER_TICK */
	disable_irq(IRQ_TIMER(TIMER_FOR_TICK));
	cxd4115_systimer_clear();
# endif /* CONFIG_LOCAL_TIMER_TICK */
}

static void resume_timer(void)
{
# ifdef CONFIG_LOCAL_TIMER_TICK
	local_timer_setup();
# else /* CONFIG_LOCAL_TIMER_TICK */
	cxd4115_systimer_clear();
	cxd4115_systimer_setup();
	cxd4115_systimer_start();
	enable_irq(IRQ_TIMER(TIMER_FOR_TICK));
# endif /* CONFIG_LOCAL_TIMER_TICK */
}
#endif /* CONFIG_PM */

struct sys_timer cxd4115_timer __initdata_refok = {
	.init		= cxd4115_init_timer,
	.offset		= cxd4115_gettimeoffset,
#ifdef CONFIG_PM
	.suspend	= suspend_timer,
	.resume		= resume_timer,
#endif
};

EXPORT_SYMBOL(read_sched_clock);
