/*
 * arch/arm/mach-cxd4105/oom_kill.c
 *
 * for printing information during OOM
 *
 * Copyright 2005,2006,2007 Sony Corporation
 */

#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/mman.h>
#include <linux/swap.h>
#include <linux/pagemap.h>
#include <linux/hugetlb.h>
#include <linux/oom_kill.h>

struct vmalloc_info {
	unsigned long	used;
	unsigned long	largest_chunk;
};

#ifdef CONFIG_MMU
#define VMALLOC_TOTAL (VMALLOC_END - VMALLOC_START)
extern void get_vmalloc_info(struct vmalloc_info *vmi);
#else

#define VMALLOC_TOTAL 0UL
#define get_vmalloc_info(vmi)			\
do {						\
	(vmi)->used = 0;			\
	(vmi)->largest_chunk = 0;		\
} while(0)

#endif

extern int task_statm(struct mm_struct *mm, int *shared, int *text, int *data, int *resident);

static char oom_buf[1024];
static int show_meminfo_and_panic(struct task_struct *target)
{
	struct sysinfo i;
	unsigned long committed;
	unsigned long allowed;
	struct vmalloc_info vmi;
	long cached;

#define K(x) ((x) << (PAGE_SHIFT - 10))
	si_meminfo(&i);
	si_swapinfo(&i);
	committed = atomic_read(&vm_committed_space);
	allowed = ((totalram_pages - hugetlb_total_pages())
		* sysctl_overcommit_ratio / 100) + total_swap_pages;

	cached = global_page_state(NR_FILE_PAGES) -
		total_swapcache_pages - i.bufferram;
        if (cached < 0)
                cached = 0;

	get_vmalloc_info(&vmi);

	/*
	 * Tagged format, for easy grepping and expansion.
	 */
	sprintf(oom_buf,
		"Out Of Memory - pid(%d) to be killed\n"
		"MemTotal:     %8lu kB\n"
		"MemFree:      %8lu kB\n"
		"Buffers:      %8lu kB\n"
		"Cached:       %8lu kB\n"
		"SwapCached:   %8lu kB\n"
		"Active:       %8lu kB\n"
		"Inactive:     %8lu kB\n"
		"HighTotal:    %8lu kB\n"
		"HighFree:     %8lu kB\n"
		"LowTotal:     %8lu kB\n"
		"LowFree:      %8lu kB\n"
		"SwapTotal:    %8lu kB\n"
		"SwapFree:     %8lu kB\n"
		"Dirty:        %8lu kB\n"
		"Writeback:    %8lu kB\n"
		"AnonPages:    %8lu kB\n"
		"Mapped:       %8lu kB\n"
		"Slab:         %8lu kB\n"
		"SReclaimable: %8lu kB\n"
                "SUnreclaim:   %8lu kB\n"
                "PageTables:   %8lu kB\n"
                "NFS_Unstable: %8lu kB\n"
                "Bounce:       %8lu kB\n"
		"CommitLimit:  %8lu kB\n"
		"Committed_AS: %8lu kB\n"
		"VmallocTotal: %8lu kB\n"
		"VmallocUsed:  %8lu kB\n"
		"VmallocChunk: %8lu kB\n",
		target->pid,
		K(i.totalram),
		K(i.freeram),
		K(i.bufferram),
		K(cached),
		K(total_swapcache_pages),
		K(global_page_state(NR_ACTIVE)),
		K(global_page_state(NR_INACTIVE)),
		K(i.totalhigh),
		K(i.freehigh),
		K(i.totalram-i.totalhigh),
		K(i.freeram-i.freehigh),
		K(i.totalswap),
		K(i.freeswap),
		K(global_page_state(NR_FILE_DIRTY)),
		K(global_page_state(NR_WRITEBACK)),
		K(global_page_state(NR_ANON_PAGES)),
		K(global_page_state(NR_FILE_MAPPED)),
		K(global_page_state(NR_SLAB_RECLAIMABLE) +
		  global_page_state(NR_SLAB_UNRECLAIMABLE)),
		K(global_page_state(NR_SLAB_RECLAIMABLE)),
		K(global_page_state(NR_SLAB_UNRECLAIMABLE)),
		K(global_page_state(NR_PAGETABLE)),
		K(global_page_state(NR_UNSTABLE_NFS)),
		K(global_page_state(NR_BOUNCE)),
		K(allowed),
		K(committed),
		(unsigned long)VMALLOC_TOTAL >> 10,
		vmi.used >> 10,
		vmi.largest_chunk >> 10
		);
#undef K

	printk(KERN_EMERG "%s", oom_buf);
	return SIGSEGV;
}

int before_oom_kill_task(struct task_struct *target)
{
	struct task_struct *g, *p;
	struct mm_struct *mm;
	char tcomm[sizeof(target->comm)];
	struct timespec uptime;
	extern unsigned long badness(struct task_struct *p, unsigned long uptime);

	do_posix_clock_monotonic_gettime(&uptime);

#define K(x) ((x) << (PAGE_SHIFT - 10))
	printk(KERN_EMERG "\n==========process info(size in KB)===============\n");
	printk(KERN_EMERG "%-12spid\tppid\tsize\trss\tshare\ttext\tdata\tbadness\n", "name");
	do_each_thread(g, p) {
		if (p->mm) {
			int size = 0, resident = 0, shared = 0, text = 0, data = 0;
			int ppid = pid_alive(p) ? p->group_leader->real_parent->tgid : 0;
			mm = get_task_mm(p);
			if (mm) {
				get_task_comm(tcomm, p);
	
				size = task_statm(mm, &shared, &text, &data, &resident);
				mmput(mm);

				printk(KERN_EMERG "%-12s%d\t%d\t%d\t%d\t%d\t%d\t%d\t%lu\n",
				       tcomm, p->pid, ppid, K(size), K(resident),
				       K(shared), K(text), K(data), badness(p, uptime.tv_sec));
			}
		}
	} while_each_thread(g, p);
	printk(KERN_EMERG "\n\n");
#undef K
	
	return show_meminfo_and_panic(target);
}

void after_oom_kill_task(struct task_struct *p)
{
	return ;
}

