/* 2008-10-21: File changed by Sony Corporation */
/*
 *  linux/arch/arm/mm/context.c
 *
 *  Copyright (C) 2002-2003 Deep Blue Solutions Ltd, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/mm.h>

#include <asm/mmu_context.h>
#include <asm/tlbflush.h>

DEFINE_SPINLOCK(cpu_asid_lock);
unsigned int cpu_last_asid = ASID_FIRST_VERSION;
cpumask_t cpu_asid_rollover = CPU_MASK_NONE;

/*
 * We fork()ed a process, and we need a new context for the child
 * to run in.  We reserve version 0 for initial tasks so we will
 * always allocate an ASID. The ASID 0 is reserved for the TTBR
 * register changing sequence.
 */
void __init_new_context(struct task_struct *tsk, struct mm_struct *mm)
{
	mm->context.id = 0;
}

void __flush_context(void)
{
	cpu_clear(smp_processor_id(), cpu_asid_rollover);

	/* set the reserved ASID before flushing the TLB */
	asm("mcr	p15, 0, %0, c13, c0, 1	@ set reserved context ID\n"
	    :
	    : "r" (0));
	isb();
	local_flush_tlb_all();
	if (icache_is_vivt_asid_tagged()) {
		asm("mcr	p15, 0, %0, c7, c5, 0	@ invalidate I-cache\n"
		    "mcr	p15, 0, %0, c7, c5, 6	@ flush BTAC/BTB\n"
		    :
		    : "r" (0));
		dsb();
	}
}

void __new_context(struct mm_struct *mm)
{
	unsigned int asid;

	asid = ++cpu_last_asid;
	if (asid == 0)
		asid = cpu_last_asid = ASID_FIRST_VERSION;

	/*
	 * If we've used up all our ASIDs, we need to start a new
	 * version and mark all the CPUs for flushing the local TLB at
	 * the current or next context switch.
	 */
	if (unlikely((asid & ~ASID_MASK) == 0)) {
		asid = ++cpu_last_asid;
		cpus_setall(cpu_asid_rollover);
	}

	mm->cpu_vm_mask = cpumask_of_cpu(smp_processor_id());
	mm->context.id = asid;
}
