/* 2007-11-13: File added by Sony Corporation */
/*
 *  arch/arm/snsc/snsc_boot_time.c
 *
 *  Copyright 2002-2007 Sony Corporation
 *
 *  This program is free software; you can redistribute	 it and/or modify it
 *  under  the terms of	 the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the	License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 *  WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO	EVENT  SHALL   THE AUTHOR  BE	 LIABLE FOR ANY	  DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 *  USE, DATA,	OR PROFITS; OR	BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/sched.h>
#include <linux/snsc_boot_time.h>

#include <asm/div64.h>

#define NAME "BootTime: "

static void
mach_count_set(struct boot_time_entry *pEntry)
{
	unsigned long long t = sched_clock();

	pEntry->count_lo = (u32)t;
	pEntry->count_hi = (u32)(t >> 32);
}

static void
mach_print(struct seq_file *m, struct boot_time_entry *pEntry)
{
	u64 dividend;

	dividend = ((u64)pEntry->count_hi << 32) | pEntry->count_lo;
	do_div(dividend, 1000000);	/* sched_clock() returns nano-sec. */
	seq_printf(m, " : %5u [ms]\n", (u32)dividend);
}

static struct boot_time_md btmd = {
	init:      NULL,
	count_set: mach_count_set,
	print:     mach_print,
};

void
mach_boot_time_init(void)
{
	boot_time_init(&btmd);
}
