/* 2007-12-11: File added by Sony Corporation */
/********************************************************************
 * Philips ISP176x Debugging and Conditinal Compilation header file
 *
 * (c) 2002 Koninklijke Philips Electronics N.V. All rights reserved. <usb.linux@philips.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * File Name: isp1761.h
 *
 * Refering linux kernel version 2.6.9
 *
 * History:
 *
 * Date               Author                  Comments
 * --------------------------------------------------------------------
 * Nov 29 2005       Prabhakar Kalasani       Initial version
 * May 5, 2006       Grant Hatamosa           Added ISO support
 * May 5, 2006       Grace Baldonasa          Added MTP support (Available only
 *                                            with Peripheral Controller code)
 *
 **********************************************************************
 */

#ifndef __ISP1761_H__
#define __ISP1761_H__

/* Platform Specifical Definitions */
#define 	HC_EDGE_INT		(1 << 1) /* edge triggle int */
#define 	HC_BUS_WD_32B		(1 << 8) /* 32 bit bus width */
#define 	ISP1761_HC_MODE		(HC_EDGE_INT | HC_BUS_WD_32B)

/* Driver tuning, per Philips requirements: */


#define MEM_TO_CHECK            4096  /*bytes, must be multiple of 2 */

/* BIT defines */
#define BIT0    (1 << 0)
#define BIT1    (1 << 1)
#define BIT2    (1 << 2)
#define BIT3    (1 << 3)
#define BIT4    (1 << 4)
#define BIT5    (1 << 5)
#define BIT6    (1 << 6)
#define BIT7    (1 << 7)
#define BIT8    (1 << 8)
#define BIT9    (1 << 9)
#define BIT10   (1 << 10)
#define BIT11   (1 << 11)
#define BIT12   (1 << 12)
#define BIT13   (1 << 13)
#define BIT14   (1 << 14)
#define BIT15   (1 << 15)
#define BIT16   (1 << 16)
#define BIT17   (1 << 17)
#define BIT18   (1 << 18)
#define BIT19   (1 << 19)
#define BIT20   (1 << 20)
#define BIT21   (1 << 21)
#define BIT22   (1 << 22)
#define BIT23   (1 << 23)
#define BIT24   (1 << 24)
#define BIT25   (1 << 26)
#define BIT27   (1 << 27)
#define BIT28   (1 << 28)
#define BIT29   (1 << 29)
#define BIT30   (1 << 30)
#define BIT31   (1 << 31)

/* Definitions Related to Little Endian & Big Endian */
#define tole(x) __constant_cpu_to_le32(x)
#define tobe(x) __constant_cpu_to_be32(x)

/* Definitions Related to Chip Address and CPU Physical Address
 * cpu_phy_add: CPU Physical Address , it uses 32 bit data per address
 * chip_add   : Chip Address, it uses double word(64) bit data per address
 */
#define chip_add(cpu_phy_add)  (cpu_phy_add-0x400)/8
#define cpu_phy_add(chip_add)  (8*chip_add)+0x400

/* for getting end add, and start add, provided we have one address with us */
/* IMPORTANT length  hex(base16) and dec(base10) works fine*/
#define end_add(start_add,length) (start_add+(length-4))
#define start_add(end_add,length) (end_add-(length-4))

/* Device Registers*/
#define DEV_UNLOCK_REGISTER             0x27C
#define DEV_INTERRUPT_REGISTER          0x218


/* The QHA Data Structure */
struct phci_qh {
	u32                 hw_info1; 	/* see QHA  W0, the first 32bits */
	u32                 hw_info2; 	/* see QHA  W1 */
	u32                 hw_info3; 	/* see QHA  W2 */
	u32                 hw_info4; 	/* see QHA  W3 */
	u32                 Reserved1; 	/* Reserved1..4. NOT in USE */
	u32                 Reserved2;
	u32                 Reserved3;
	u32                 Reserved4;
}phci_qh  __attribute__ ((aligned (32)));


/*
 *  We have hardware support for asynchronous DMA transfers.  We can use PIO
 *  mode as well.  The chips registers are mapped as follows:
 *
 *  Base address for pio mode: ioremap_nocache(PXA_CS2_BASE,PAGE_SIZE);
 *  Physical addresses for DMA transfers:
 *  Note also that these are normally defined in the platform specific header
 *  files.  DON'T define them within your driver, that's extremely non-portable.
 */
typedef struct isp1761_hal {
	void    *mem_base; 	/* Memory base */
	__u8        irq_usage;  /* NUmber of drivers using INT channel */
} isp1761_hal_t;

typedef struct {
	char *name;             	/* dma stream identifier */
	char *data_buffer;              /* pointer to in memory dma buffer (virtual) */
	char *phys_data_buffer;         /* pointer to in memory dma buffer (physical) */
	char *trash_buffer;             /* don't ask... */
	char *phys_trash_buffer;
	u_int dma_ch;                   /* DMA channel number */

	/* the dma request channel to use, not to be confused with the above */
	volatile u32 *drcmr;
	u_long dcmd;                   	/* DMA dcmd field */
	u_long dev_addr;                /* device physical address for this channel */
	u_long blklen;                  /* block length of specific transfer */
	char output;                    /* 0 for input, 1 for ouput */
} isp1761_channel_t;

typedef struct {
	isp1761_channel_t *dreq0;
	isp1761_channel_t *dreq1;
	struct semaphore sem;       /* may be handy for races... */
} dma_state_t;


//------------------------------




/* Values for id_flags filed of isp1761_driver_t */
#define         ISP1761_HC                     	0 	/* Host Controller Driver */
#define         ISP1761_DC                     	1       /* Device Controller Driver */
#define         ISP1761_OTG                    	2       /* Otg Controller Driver */
#define         ISP1761_LAST_DEV               	(ISP1761_OTG+1)
#define         ISP1761_1ST_DEV                	(ISP1761_HC)

#define         HC_SPARAMS_REG                 	0x04    /* Structural Parameters Register */
#define         HC_CPARAMS_REG                 	0x08    /* Capability Parameters Register */

#define         HC_USBCMD_REG                  	0x20    /* USB Command Register */
#define         HC_USBSTS_REG                  	0x24    /* USB Status Register */
#define         HC_INTERRUPT_REG_EHCI          	0x28    /* INterrupt Enable Register */
#define         HC_FRINDEX_REG                 	0x2C    /* Frame Index Register */

#define         HC_CONFIGFLAG_REG              	0x60    /* Conigured Flag  Register */
#define         HC_PORTSC1_REG                 	0x64    /* Port Status Control for Port1 */

/*ISO Transfer Registers */
#define         HC_ISO_PTD_DONEMAP_REG         	0x130   /* ISO PTD Done Map Register */
#define         HC_ISO_PTD_SKIPMAP_REG         	0x134   /* ISO PTD Skip Map Register */
#define         HC_ISO_PTD_LASTPTD_REG         	0x138   /* ISO PTD Last PTD Register */

/*INT Transfer Registers */
#define         HC_INT_PTD_DONEMAP_REG         	0x140   /* INT PTD Done Map Register */
#define         HC_INT_PTD_SKIPMAP_REG         	0x144   /* INT PTD Skip Map Register */
#define         HC_INT_PTD_LASTPTD_REG         	0x148   /* INT PTD Last PTD Register  */

/*ATL Transfer Registers */
#define         HC_ATL_PTD_DONEMAP_REG         	0x150   /* ATL PTD Last PTD Register  */
#define         HC_ATL_PTD_SKIPMAP_REG         	0x154   /* ATL PTD Last PTD Register  */
#define         HC_ATL_PTD_LASTPTD_REG         	0x158   /* ATL PTD Last PTD Register  */

/*General Purpose Registers */
#define         HC_HW_MODE_REG                 	0x300   /* H/W Mode Register  */
#define         HC_CHIP_ID_REG                 	0x304   /* Chip ID Register */
#define         HC_SCRATCH_REG                 	0x308   /* Scratch Register */
#define         HC_RESET_REG                   	0x30C   /* HC Reset Register */

/* Interrupt Registers */
#define         HC_INTERRUPT_REG               	0x310   /* Interrupt Register */
#define         HC_INTENABLE_REG               	0x314   /* Interrupt enable Register */
#define         HC_ISO_IRQ_MASK_OR_REG         	0x318   /* ISO Mask OR Register */
#define         HC_INT_IRQ_MASK_OR_REG         	0x31C   /* INT Mask OR Register*/
#define         HC_ATL_IRQ_MASK_OR_REG         	0x320   /* ATL Mask OR Register */
#define         HC_ISO_IRQ_MASK_AND_REG        	0x324   /* ISO Mask AND Register */
#define         HC_INT_IRQ_MASK_AND_REG        	0x328   /* INT Mask AND Register */
#define         HC_ATL_IRQ_MASK_AND_REG        	0x32C   /* ATL Mask AND Register */

/*RAM Registers */
#define         HC_DMACONFIG_REG               	0x330   /* DMA Config Register */
#define         HC_TRANS_COUNT_REG             	0x334   /* Transfer Counter Register */
#define         HC_ATL_TIMEOUT_REG             	0x338   /* ATL Timeout Register */
#define         HC_MEM_READ_REG                	0x33C   /* Memory Register */

/*interrupt count and buffer status register*/

#define         HC_BUFFER_STATUS_REG           	0x334   /* Buffer status Register */
#define         HC_INT_THRESHOLD_REG           	0x340
#define         HC_OTG_INTERRUPT               	0x400

/* Deveice Controller Registers */
#define         DC_CHIP_ID_REG                	0x270   /* DC Chip ID Register */


/* OTG Controller Registers*/
#define 	OTG_PID_VID_REG			0x370   /* Product ID and Version ID*/
#define 	OTG_CONTROL_REG			0x374   /* OTG Control Register*/




/* OTG Register Setup Value */
/* Set OTG Port to Host Controller Mode, see isp1761 datasheet, P91, 9.5 */
#define 	OTG_PORT_HOST_MODE		0x00800018
#define 	OTG_PORT_SLAVE_MODE		0x00060400
#define 	OTG_PORT_OTG_MODE		0x00000086

#ifdef CONFIG_SNSC_ISP1761_PORT1_OTG
#define		OTG_PORT_MODE		OTG_PORT_OTG_MODE
#endif

#ifdef CONFIG_SNSC_ISP1761_PORT1_PC
#define		OTG_PORT_MODE		OTG_PORT_SLAVE_MODE
#endif

#ifdef CONFIG_SNSC_ISP1761_PORT1_HC
#define		OTG_PORT_MODE		OTG_PORT_HOST_MODE
#endif


struct isp1761_driver;
typedef struct _isp1761_id {
	__u16 idVendor;
	__u16 idProduct;
	unsigned long driver_info;
}isp1761_id;

typedef struct isp1761_dev {
	/*added for pci device*/
	struct device *dev;
	struct  isp1761_driver *driver; /* which driver has allocated this device */
	void            *driver_data;   /* data private to the host controller driver */
	void            *otg_driver_data; /*data private for otg controler*/
	unsigned char   index;          /* local controller (HC/DC/OTG) */
	unsigned int    irq;    /*Interrupt Channel allocated for this device */
	void (*handler)(struct isp1761_dev *dev,
			void *isr_data,
			struct pt_regs *);      /* Interrupt Serrvice Routine */
	void            *isr_data;              /* isr data of the driver */
	unsigned long   int_reg;                /* Interrupt register */
	unsigned long   alt_int_reg;            /* Interrupt register 2*/
	unsigned long   start;
	unsigned long   length;
	struct resource *mem_res;
	unsigned long   io_base;/* Start Io address space for this device */
	unsigned long   io_len; /* IO address space length for this device */

	unsigned long   chip_id; /* Chip Id */

	char            name[80];       /* device name */
	int             active;                         /* device status */

	/* DMA resources should come TODO */
	unsigned long   dma;
	u8 *baseaddress;  /*base address for i/o ops*/
	u8 *dmabase;
}isp1761_dev_t ;


typedef struct isp1761_driver {
	char            *name;
	unsigned long   index;  	/* HC or DC or OTG */
	isp1761_id      *id;    	/*device ids*/

	/* New device inserted */
	int (*probe)(struct isp1761_dev *dev, isp1761_id *id);
	/* Device removed (NULL if not a hot-plug capable driver) */
	void (*remove)(struct isp1761_dev *dev);

#ifdef CONFIG_PM
	void (*suspend)(struct isp1761_dev *dev);       /* Device suspended */
	void (*resume)(struct isp1761_dev *dev);        /* Device woken up */
#endif /* CONFIG_PM */

} isp_1761_driver_t;

#ifdef CONFIG_SNSC_ISP1761_PORT1_OTG
static struct usb_device * phci_register_otg_device(struct isp1761_dev *dev);
/*otg exported function from host*/
static int phci_suspend_otg_port(struct isp1761_dev *dev, u32 command);
static int phci_resume_otg_port(struct isp1761_dev *dev, u32 command);
static int phci_enumerate_otg_port(struct isp1761_dev *dev, u32 command);
#endif

static int isp1761_register_driver(struct isp1761_driver *drv);
static void isp1761_unregister_driver(struct isp1761_driver *drv);

static __u32 isp1761_reg_read32(isp1761_dev_t *dev,__u16 reg,__u32 data);
static void isp1761_reg_write32(isp1761_dev_t *dev,__u16 reg,__u32 data);

static int isp1761_mem_read(isp1761_dev_t *dev, __u32 start_add,
		__u32 end_add, __u32 * buffer, __u32 length, __u16 dir);
static int isp1761_mem_write(isp1761_dev_t *dev, __u32 start_add,
		__u32 end_add, __u32 * buffer, __u32 length, __u16 dir);



/*******************START HOST CONTROLLER********************************/
/* Host controller conditional defines*/

/* For Iso support */

#ifdef CONFIG_SNSC_ISP1761_ISO_SUPPORT

#define ISO_DBG_ENTRY FALSE
#define ISO_DBG_EXIT FALSE
#define ISO_DBG_ADDR TRUE
#define ISO_DBG_DATA TRUE
#define ISO_DBG_ERR  TRUE
#define ISO_DBG_INFO TRUE

#if 0 /* Set to 1 to enable isochronous debugging */
#define iso_dbg(category, format, arg...) \
	do \
{ \
	if(category) \
	{ \
		printk(format, ## arg); \
	} \
} while(0)
#else
#define iso_dbg(category, format, arg...) while(0)
#endif

#endif /* CONFIG_SNSC_ISP1761_ISO_SUPPORT */

/*Debug For Entry/Exit of the functions */
#define HCD_DEBUG_LEVEL1
#undef HCD_DEBUG_LEVEL1
#ifdef HCD_DEBUG_LEVEL1
#define pehci_entry(format, args... ) printk(format, ##args)
#else
#define pehci_entry(format, args...) do { } while(0)
#endif

/*Debug for Port Info and Errors */
#define HCD_DEBUG_LEVEL2
#undef HCD_DEBUG_LEVEL2
#ifdef HCD_DEBUG_LEVEL2
#define pehci_print(format, args... ) printk(format, ##args)
#else
#define pehci_print(format, args...) do { } while(0)
#endif

/*Debug For the Port changes and Enumeration */
#define HCD_DEBUG_LEVEL3
#undef HCD_DEBUG_LEVEL3
#ifdef HCD_DEBUG_LEVEL3
#define pehci_info(format,arg...) printk(format, ##arg)
#else
#define pehci_info(format,arg...) do {} while (0)
#endif

/*Debug For Transfer flow  */
#define HCD_DEBUG_LEVEL4
#undef HCD_DEBUG_LEVEL4
#ifdef HCD_DEBUG_LEVEL4
#define pehci_check(format,args...) printk(format, ##args)
#else
#define pehci_check(format,args...)
#endif
/*******************END HOST CONTROLLER**********************************/



/*******************START DEVICE CONTROLLER******************************/

/* For MTP support */
#undef MTP_ENABLE /* Enable to add MTP support on ISP1761 Device Controller
		   * Requires MTP class driver
		   */

/*Debug Entery/Exit of Function as well as some other Information(I'm not sure)*/
#define DEV_DEBUG_LEVEL2
#undef DEV_DEBUG_LEVEL2
#ifdef DEV_DEBUG_LEVEL2
#define dev_print(format,arg...) printk(format, ##arg)
#else
#define dev_print(format,arg...) do {} while (0)
#endif

/*Debug for Interrupt , Registers , Device Enable/Disable and some other info */
#define DEV_DEBUG_LEVEL3
#undef DEV_DEBUG_LEVEL3
#ifdef DEV_DEBUG_LEVEL3
#define dev_inform(format,arg...) printk(format, ##arg)
#else
#define dev_inform(format,arg...) do {} while (0)
#endif

/*Debug for Tranffer flow , Enumeration and Packet info */
#define DEV_DEBUG_LEVEL4
#undef DEV_DEBUG_LEVEL4
#ifdef DEV_DEBUG_LEVEL4
#define dev_check(format,args...) printk(format, ##args)
#else
#define dev_check(format,args...) do{}while(0)
#endif
/*******************END DEVICE CONTROLLER********************************/


/*******************START MSCD*******************************************/
/*Debug Entery/Exit of Function as well as some other Information(I'm not sure)*/
#undef MSCD_DEBUG_LEVEL2
#ifdef MSCD_DEBUG_LEVEL2
#define mscd_print(format,arg...) printk(format, ##arg)
#else
#define mscd_print(format,arg...) do {} while (0)
#endif

/*Debug for Info */
#undef MSCD_DEBUG_LEVEL3
#ifdef MSCD_DEBUG_LEVEL3
#define mscd_info(format,arg...) printk(format, ##arg)
#else
#define mscd_info(format,arg...) do {} while (0)
#endif
/*******************END MSCD*********************************************/


/*******************START OTG CONTROLLER*********************************/
/*Debug for Entry/Exit and Info */
#undef OTG_DEBUG_LEVEL1
#ifdef OTG_DEBUG_LEVEL1
#define otg_entry(format, args... ) printk(format, ##args)
#else
#define otg_entry(format, args...) do { } while(0)
#endif

/*Debug for State Machine Flow */
#undef OTG_DEBUG_LEVEL2
#ifdef OTG_DEBUG_LEVEL2
#define otg_print(format,arg...) printk(format, ##arg)
#else
#define otg_print(format,arg...) do {} while (0)
#endif
/*Debug for Info */
#undef OTG_DEBUG_LEVEL3
#ifdef OTG_DEBUG_LEVEL3
#define otg_info(format,arg...) printk(format, ##arg)
#else
#define otg_info(format,arg...) do {} while (0)
#endif
/*******************END OTG CONTROLLER***********************************/



/*******************START FOR HAL ***************************************/
/*Debug For Entry and Exit of the functions */
#define HAL_DEBUG_LEVEL1
#undef HAL_DEBUG_LEVEL1
#ifdef HAL_DEBUG_LEVEL1
#define hal_entry(format, args... ) printk(format, ##args)
#else
#define hal_entry(format, args...) do { } while(0)
#endif

/*Debug For Interrupt information */
#define HAL_DEBUG_LEVEL2
#undef HAL_DEBUG_LEVEL2
#ifdef HAL_DEBUG_LEVEL2
#define hal_int(format, args... ) printk(format, ##args)
#else
#define hal_int(format, args...) do { } while(0)
#endif

/*Debug For HAL Initialisation and Mem Initialisation */
#define HAL_DEBUG_LEVEL3
#undef HAL_DEBUG_LEVEL3
#ifdef HAL_DEBUG_LEVEL3
#define hal_init(format, args... ) printk(format, ##args)
#else
#define hal_init(format, args...) do { } while(0)
#endif
/*******************END FOR HAL*******************************************/



/*******************START FOR ALL CONTROLLERS*****************************/
#undef CONFIG_USB_OTG   /*undef for Host only and Device only */

#ifdef CONFIG_USB_DEBUG
#define DEBUG
#else
#undef DEBUG
#endif
/*******************END FOR ALL CONTROLLERS*******************************/


#endif /* __ISP1761_H__ */
