/*-----------------------------------------------------------------------------
 * drivers/usb/mcd/usb_mcd_types.h
 * 
 * USB controller driver
 * 
 * Copyright 2009 Sony Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _USB_MCD_ACCESS_H_
#define _USB_MCD_ACCESS_H_

#define MCD_READ8(_pBase, _offset) *(volatile uint8_t*)(_pBase + _offset)
#define MCD_READ16(_pBase, _offset) *(volatile uint16_t*)(_pBase + _offset)
#define MCD_READ32(_pBase, _offset) *(volatile uint32_t*)(_pBase + _offset)

#define MCD_WRITE8(_pBase, _offset, _data) { \
    DBG(2, "Write8(%p, %x, %02x)\n", _pBase, _offset, _data); \
    *(volatile uint8_t*)(_pBase + _offset) = _data; \
}
#if 1				/* TEST */
#define MCD_WRITE16(_pBase, _offset, _data) { \
    DBG(2, "Write16(%p, %x, %04x)\n", _pBase, _offset, _data); \
    *(volatile uint16_t*)(_pBase + _offset) = _data; \
}
#else
#define MCD_WRITE16(_pBase, _offset, _data) { \
    printk("Write16(%p, %x, %04x)\n", _pBase, _offset, _data); \
    *(volatile uint16_t*)(_pBase + _offset) = _data; \
}
#endif
#define MCD_WRITE32(_pBase, _offset, _data) { \
    DBG(2, "Write32(%p, %x, %08x)\n", _pBase, _offset, _data); \
    *(volatile uint32_t*)(_pBase + _offset) = _data; \
}

/* indexed vs. flat register model */
#ifdef MCD_FLAT_REG
#define mcd_select_end(_pBase, _bEnd)
#define MCD_READ_CSR8(_pBase, _bOffset, _bEnd) \
    MCD_READ8(_pBase, MCD_END_OFFSET(_bEnd, _bOffset))
#define MCD_READ_CSR16(_pBase, _bOffset, _bEnd) \
    MCD_READ16(_pBase, MCD_END_OFFSET(_bEnd, _bOffset))
#define MCD_WRITE_CSR8(_pBase, _bOffset, _bEnd, _bData) \
    MCD_WRITE8(_pBase, MCD_END_OFFSET(_bEnd, _bOffset), _bData)
#define MCD_WRITE_CSR16(_pBase, _bOffset, _bEnd, _bData) \
    MCD_WRITE16(_pBase, MCD_END_OFFSET(_bEnd, _bOffset), _bData)
#else
#define mcd_select_end(_pBase, _bEnd) \
    MCD_WRITE8(_pBase, MCD_O_HDRC_INDEX, _bEnd)
#define MCD_READ_CSR8(_pBase, _bOffset, _bEnd) \
    MCD_READ8(_pBase, (_bOffset + 0x10))
#define MCD_READ_CSR16(_pBase, _bOffset, _bEnd) \
    MCD_READ16(_pBase, (_bOffset + 0x10))
#define MCD_WRITE_CSR8(_pBase, _bOffset, _bEnd, _bData) \
    MCD_WRITE8(_pBase, (_bOffset + 0x10), _bData)
#define MCD_WRITE_CSR16(_pBase, _bOffset, _bEnd, _bData) \
    MCD_WRITE16(_pBase, (_bOffset + 0x10), _bData)
#endif

/* flat only register model */
#define MCD_READ_OCSR8(_pBase, _bOffset, _bEnd) \
    MCD_READ8(_pBase, MCD_END_OFFSET(_bEnd, _bOffset))
#define MCD_READ_OCSR16(_pBase, _bOffset, _bEnd) \
    MCD_READ16(_pBase, MCD_END_OFFSET(_bEnd, _bOffset))
#define MCD_WRITE_OCSR8(_pBase, _bOffset, _bEnd, _bData) \
    MCD_WRITE8(_pBase, MCD_END_OFFSET(_bEnd, _bOffset), _bData)
#define MCD_WRITE_OCSR16(_pBase, _bOffset, _bEnd, _bData) \
    MCD_WRITE16(_pBase, MCD_END_OFFSET(_bEnd, _bOffset), _bData)

#endif			/* _USB_MCD_ACCESS_H_ */
