/*--------------------------------------------------------------------------*
 * drivers/usb/mcd/usb_mcd_board_cxd41xx.c
 * 
 * USB controller driver
 * 
 * Copyright 2009 Sony Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/list.h>
#include <linux/proc_fs.h>
#include <linux/platform_device.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/arch/regs-timer.h>
#include <asm/arch/regs-clk.h>
#include <linux/usb.h>
#include <linux/usb/ch9.h>
#include <linux/usb_gadget.h>
#include <linux/usb/mcd/usb_mcd.h>
#include <linux/usb/mcd/usb_otg_control.h>
#include "../core/hub.h"
#include "../core/hcd.h"

#include "usb_mcd_cnf.h"
#include "usb_mcd_debug.h"
#include "usb_mcd_dma.h"
#include "usb_mcd_otg.h"
#include "usb_mcd_gadget.h"
#include "usb_mcd_host.h"
#include "usb_mcd_device.h"
#include "usb_mcd_access.h"
#include "usb_mcd_musbhdrc.h"

/*  ON/OFF */
#ifdef ON
#undef ON
#endif
#define ON	1
#ifdef OFF
#undef OFF
#endif
#define OFF	0

/************************* system dependenced definitions *********************/

/* MISC/GPIOEASY3 saved value for suspend/resume */
struct _cxd41xx_suspend_save {
	u8 clock;
	u8 iddig;
	u8 vbusin;
	u8 vcmpen;
	u8 suspend;
	u8 idpullup;
};

#if defined(CONFIG_ARCH_CXD4105)
#include <asm/arch/cxd4105_gpio.h>
#include "usb_mcd_board_cxd4105.h"
#else /* CONFIG_ARCH_CXD4115 etc. */
#include <asm/arch/regs-misc.h>
#include <asm/arch/cxd4115_gpio.h>
#include "usb_mcd_board_cxd4115.h"
#endif

/************************** macro, variable definitions ***********************/

/* Port descprocե̾ */
#define PORT_DESC_PROCFILE	"usb_mcd_port_desc"

/* Pѥ᡼ѹprocե̾ */
#define PPARAM_PROCFILE		"usb_mcd_pparam"
#define PPARAM_WRITE_BUFSIZE	(BOARD_PPARAM_NUM * 3 + 4)

/* Port Number */
#define MCD_MAX_PORT_NUM 2
#define MCD_MAX_SELECT_PORT_NUM 1

/********************************* resources *********************************/

/* mcd_resources, mcd_dma_mask, mcd_dma_mask ... ǥХ */

static void nop_release(struct device *dev);

static struct resource mcd_resources[] = {
	[0] = {
	       .start = ADR_START,
	       .end = ADR_END,
	       .flags = IORESOURCE_IO,
	       },
	[1] = {
	       .name = "USB_HS",
	       .start = IRQ_USB0,
	       .end = 0,
	       .flags = IORESOURCE_IRQ,
	       },
	[2] = {
	       .name = "USB_DMAC",
	       .start = IRQ_USB1,
	       .end = 0,
	       .flags = IORESOURCE_IRQ,
	       },
};

static u64 mcd_dma_mask = 0xffffffffUL;

struct platform_device mcd_platform_device = {
	.name = "usb_mcd",
	.id = 0,
	.dev = {
		.dma_mask = &mcd_dma_mask,
		.coherent_dma_mask = 0xffffffffUL,
		.release = nop_release,
		},
	.num_resources = ARRAY_SIZE(mcd_resources),
	.resource = mcd_resources,
};

static u8 pparam[BOARD_PPARAM_NUM];

/***************************** struct definitions *****************************/

/* CID type */
enum _cxd41xx_cid_type {
	_CXD41XX_CID_TYPE_NONE = 0,
	_CXD41XX_CID_TYPE_PHY,
	_CXD41XX_CID_TYPE_FIX,
	_CXD41XX_CID_TYPE_GPIO,
};

/* Vbus type */
enum _cxd41xx_vbus_type {
	_CXD41XX_VBUS_TYPE_NONE = 0,
	_CXD41XX_VBUS_TYPE_PHY,
	_CXD41XX_VBUS_TYPE_FIX,
	_CXD41XX_VBUS_TYPE_GPIO,
	_CXD41XX_VBUS_TYPE_GPIOX,
	_CXD41XX_VBUS_TYPE_PHYPRV,
};

/* GPIO information structure */
struct _cxd41xx_gpio {
	gpio_portid prt;
	gpio_bitnum bit;
};

/* GPIO information for selection of USB port */
static struct _cxd41xx_select_port {
	struct _cxd41xx_gpio sel;
} cxd41xx_select_port[MCD_MAX_SELECT_PORT_NUM];

/* USB port information */
static struct _cxd41xx_port {
	enum _cxd41xx_cid_type cid_type;
	enum _cxd41xx_vbus_type vbus_type;
	struct _cxd41xx_gpio cid;
	struct _cxd41xx_gpio vbus;
	u32 select_pattern;
} cxd41xx_port[MCD_MAX_PORT_NUM];

/* USB port control information */
static struct _cxd41xx_port_control {
	u32 port_num;
	u32 select_port_num;
	u32 current_port;
	struct _cxd41xx_gpio oc;
	struct _cxd41xx_port *port;
	struct _cxd41xx_select_port *select_port;
	struct _cxd41xx_suspend_save save;
} cxd41xx_port_control = {
	.port		= cxd41xx_port,
	.select_port	= cxd41xx_select_port,
};

static struct usb_otg_control *cxd41xx_otgc;

/***************************** internal functions *****************************/

/* dma dummy release */
static void nop_release(struct device *dev)
{
	DBG(3, "%s %s\n", __FUNCTION__, dev->bus_id);
}

#if defined(CONFIG_USB_MCD_HOST)
/* det_oc ... GPIO interrupt callback to detect OverCurrent */
static irqreturn_t det_oc(int irq, void *param)
{
#if defined(CONFIG_ARCH_CXD4105)
	struct _cxd41xx_port_control *port_control = &cxd41xx_port_control;

	DBG(2, "<==\n");

	gpio_clr_intr_st_bit(port_control->oc.prt, port_control->oc.bit);
#endif				/* defined(CONFIG_ARCH_CXD4105) */

	mcd_otg_control_notify_vbuserror(cxd41xx_otgc);
	DBG(2, "==>\n");

	return IRQ_HANDLED;
}
#endif				/* defined(CONFIG_USB_MCD_HOST) */

/* det_vbuscid ... GPIO interrupt callback to change Vbus/CID level */
static irqreturn_t det_vbuscid(int irq, void *param)
{
	struct mcd_otg_intdat *otgc_intdat;
#if defined(CONFIG_ARCH_CXD4105)
	struct _cxd41xx_gpio *gpio = (struct _cxd41xx_gpio *)param;

	DBG(2, "<==\n");

	if (gpio) {
		gpio_clr_intr_st_bit(gpio->prt, gpio->bit);
	}
#endif				/* defined(CONFIG_ARCH_CXD4105) */

	otgc_intdat = otgc_to_otgi(cxd41xx_otgc);

	if (timer_pending(&otgc_intdat->detector)) {
		return IRQ_HANDLED;
	}
	mod_timer(&otgc_intdat->detector, jiffies);
	DBG(2, "==>\n");

	return IRQ_HANDLED;
}

#if defined(CONFIG_USB_MCD_HOST)
/* set_gpio_func ... set GPIO function */
static int set_gpio_func(gpio_portid prt, gpio_bitnum bit, gpio_mode mode)
{
	int ret;

	DBG(2, "<==\n");

	ret = gpio_register_bit(prt, bit);
	if (ret < 0) {
		return ret;
	}

	ret = gpio_set_mode_bit(prt, bit, mode);

	gpio_unregister_bit(prt, bit);

	DBG(2, "==>\n");
	return ret;
}
#endif				/* defined(CONFIG_USB_MCD_HOST) */

/**************************** check functions *****************************/

/*
 * ؿcheck_cid_phy
 * ؿƤӽФ롣GPIOEASY3˸CID֤ͤ
 */
static int check_cid_phy(struct usb_otg_control *otg_control, u32 pn)
{
	/* Read IDDIG */
	return read_iddig();
}

/*
 * ؿcheck_cid_fix
 * ؿƤӽФ롣b-deviceθ֤ͤ
 */
static int check_cid_fix(struct usb_otg_control *otg_control, u32 pn)
{
	return 1;
}

/*
 * ؿcheck_cid_gpio
 * ؿƤӽФ롣GPIOͤɤ߽Ф֤
 */
static int check_cid_gpio(struct usb_otg_control *otg_control, u32 pn)
{
	struct _cxd41xx_port_control *port_control = &cxd41xx_port_control;
	gpio_bit data;

	gpio_get_data_bit(port_control->port[pn].cid.prt,
			  port_control->port[pn].cid.bit, &data);

	return (int)data;
}

/*
 * ؿcheck_vbus_phy
 * ؿƤӽФ롣GPIOEASY3˸Vbus֤ͤ
 */
static enum usb_otg_vbus_stat
check_vbus_phy(struct usb_otg_control *otg_control, u32 pn)
{
	/* Vbus GPIOФʤGPIOEASY3ˤVbus */
	return read_vbusin();
}

/*
 * ؿcheck_vbus_fix
 * ؿƤӽФ롣Vbus Sessθ֤ͤ
 */
static enum usb_otg_vbus_stat
check_vbus_fix(struct usb_otg_control *otg_control, u32 pn)
{
	enum usb_otg_vbus_stat vbus_stat;

	vbus_stat = USB_OTG_VBUS_STAT_SESS;

	return vbus_stat;
}

/*
 * ؿcheck_vbus_gpio
 * ؿƤӽФ롣GPIO˸Vbus֤ͤ
 */
static enum usb_otg_vbus_stat
check_vbus_gpio(struct usb_otg_control *otg_control, u32 pn)
{
	struct _cxd41xx_port_control *port_control = &cxd41xx_port_control;
	int ret;
	gpio_bit data;
	enum usb_otg_vbus_stat vbus_stat;

	ret = gpio_get_data_bit(port_control->port[pn].vbus.prt,
				port_control->port[pn].vbus.bit, &data);
	if (ret < 0) {
		DBG(2, "GPIO read error %d\n", data);
		/* Error */
	}

	if (data) {
		vbus_stat = USB_OTG_VBUS_STAT_SESS;
	} else {
		vbus_stat = USB_OTG_VBUS_STAT_OFF;
	}

	return vbus_stat;
}

/*
 * ؿcheck_vbus_gpiox
 * ؿƤӽФ롣GPIO˸Vbusͤȿž֤
 */
static enum usb_otg_vbus_stat
check_vbus_gpiox(struct usb_otg_control *otg_control, u32 pn)
{
	struct _cxd41xx_port_control *port_control = &cxd41xx_port_control;
	int ret;
	gpio_bit data;
	enum usb_otg_vbus_stat vbus_stat;

	ret = gpio_get_data_bit(port_control->port[pn].vbus.prt,
				port_control->port[pn].vbus.bit, &data);
	if (ret < 0) {
		DBG(2, "GPIO read error %d\n", data);
		/* Error */
	}

	if (data) {
		vbus_stat = USB_OTG_VBUS_STAT_OFF;
	} else {
		vbus_stat = USB_OTG_VBUS_STAT_SESS;
	}

	return vbus_stat;
}

/*
 * ؿcheck_suspend
 * ǥХΥڥɾ֤å롣
 */
static bool check_suspend(struct usb_otg_control *otg_control)
{
	/* Read SUSPEND# */
	return read_suspend();
}

/* CIDåؿ */
static int
(*check_cid_funcs[]) (struct usb_otg_control *, u32) = {
	NULL,			/* _CXD41XX_CID_TYPE_NONE */
	check_cid_phy,		/* _CXD41XX_CID_TYPE_PHY */
	check_cid_fix,		/* _CXD41XX_CID_TYPE_FIX */
	check_cid_gpio,		/* _CXD41XX_CID_TYPE_GPIO */
};

/* Vbusåؿ */
static enum usb_otg_vbus_stat
(*check_vbus_funcs[]) (struct usb_otg_control *, u32) = {
	NULL,			/* _CXD41XX_VBUS_TYPE_NONE */
	check_vbus_phy,		/* _CXD41XX_VBUS_TYPE_PHY */
	check_vbus_fix,		/* _CXD41XX_VBUS_TYPE_FIX */
	check_vbus_gpio,	/* _CXD41XX_VBUS_TYPE_GPIO */
	check_vbus_gpiox,	/* _CXD41XX_VBUS_TYPE_GPIOX */
	check_vbus_phy,		/* _CXD41XX_VBUS_TYPE_PHYPRV */
};

static void set_attr_all_int_port(struct mcd_otg_intdat *otgc_intdat,
				  struct _cxd41xx_port_control *port_control)
{
	u32 i;
	struct _cxd41xx_port *port;
	u32 attr;

	attr = MCD_OTG_ATTR_ALL_INT_PORT;

	for (i = 0; i < port_control->port_num; i++) {
		port = &port_control->port[i];
		if (!((port->vbus_type == _CXD41XX_VBUS_TYPE_FIX) ||
		      ((port->vbus_type == _CXD41XX_VBUS_TYPE_GPIO ||
			port->vbus_type == _CXD41XX_VBUS_TYPE_GPIOX) &&
		       check_gpio_sys(port->vbus.prt)) ||
		      (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY ||
		       port->vbus_type == _CXD41XX_VBUS_TYPE_PHYPRV)) ||
		    !((port->cid_type == _CXD41XX_CID_TYPE_FIX) ||
		      (port->cid_type == _CXD41XX_CID_TYPE_GPIO &&
		       check_gpio_sys(port->cid.prt)))) {
			attr = 0;
			break;
		}
	}
	otgc_intdat->attributes |= attr;
}

static void set_attr_using_phy_vbus(struct mcd_otg_intdat *otgc_intdat,
				    struct _cxd41xx_port_control *port_control)
{
	u32 i;
	struct _cxd41xx_port *port;

	for (i = 0; i < port_control->port_num; i++) {
		port = &port_control->port[i];
		if (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY ||
		    port->vbus_type == _CXD41XX_VBUS_TYPE_PHYPRV) {
			otgc_intdat->attributes |= MCD_OTG_ATTR_USING_PHY_VBUS;
			return;
		}
	}
}

static void set_attr_using_phy_cid(struct mcd_otg_intdat *otgc_intdat,
				   struct _cxd41xx_port_control *port_control)
{
	u32 i;
	struct _cxd41xx_port *port;

	for (i = 0; i < port_control->port_num; i++) {
		port = &port_control->port[i];
		if (port->cid_type == _CXD41XX_CID_TYPE_PHY) {
			otgc_intdat->attributes |= MCD_OTG_ATTR_USING_PHY_CID;
			return;
		}
	}
}

static inline bool check_using_phy_vbus(struct mcd_otg_intdat *otgc_intdat)
{
	return otgc_intdat->attributes & MCD_OTG_ATTR_USING_PHY_VBUS;
}

static inline bool check_using_phy_cid(struct mcd_otg_intdat *otgc_intdat)
{
	return otgc_intdat->attributes & MCD_OTG_ATTR_USING_PHY_CID;
}

/**************************** operation functions *****************************/

/* request_vbuscid_irq - vbus⤷cidγ߽ */
inline static int request_vbuscid_irq(struct _cxd41xx_gpio *gpio)
{
	int ret = 0;
	const char devname[] = "usb_mcd_cxd41xx_vbuscid";

	if (check_gpio_sys(gpio->prt)) {
		/* Vbusг */
#if defined(CONFIG_ARCH_CXD4105)
		ret = request_irq(IRQ_GPIOx(gpio->bit), det_vbuscid,
				  IRQF_DISABLED, devname, gpio);
		if (ret < 0) {
			return ret;
		}
		ret = gpio_set_intr_trig_bit(gpio->prt, gpio->bit, GPIO_TRG_EDGE);
		if (ret < 0) {
			return ret;
		}
		ret = gpio_ctrl_intr_en_bit(gpio->prt, gpio->bit, 1);
		if (ret < 0) {
			return ret;
		}
#else
		ret = request_irq(IRQ_GPIO_RISE(gpio->bit), det_vbuscid,
				  IRQF_DISABLED, devname, gpio);
		if (ret < 0) {
			return ret;
		}
		ret = request_irq(IRQ_GPIO_FALL(gpio->bit), det_vbuscid,
				  IRQF_DISABLED, devname, gpio);
		if (ret < 0) {
			return ret;
		}
#endif				/* defined(CONFIG_ARCH_CXD4105) */
	}

	return 0;
}

/*
 *	control_ioƤӽФؿ
 */
static int start_port(struct usb_otg_control *otg_control,
		      struct _cxd41xx_port *port)
{
	int ret = 0;
	gpio_mode mode = 0;

	/* VBUS */
	if (port->vbus_type == _CXD41XX_VBUS_TYPE_GPIO
	    || port->vbus_type == _CXD41XX_VBUS_TYPE_GPIOX) {
		ret = gpio_register_bit(port->vbus.prt,	port->vbus.bit);
		if (ret < 0) {
			return ret;
		}
#if defined(CONFIG_ARCH_CXD4105)
		mode = port->vbus_type == _CXD41XX_VBUS_TYPE_GPIO
			? GPIOMODE_PULLDOWN : GPIOMODE_PULLUP;
#endif				/* defined(CONFIG_ARCH_CXD4105) */
		ret = gpio_set_mode_bit(port->vbus.prt, port->vbus.bit,
					GPIOMODE_INPUT | mode);
		if (ret < 0) {
			return ret;
		}
		ret = request_vbuscid_irq(&port->vbus);
		if (ret < 0) {
			return ret;
		}
	}
#if defined(CONFIG_ARCH_CXD4105)
	else if (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY) {
		control_vcmpen_on();
		control_vbusin_thru();
	}
#else
	else if (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY ||
		 port->vbus_type ==_CXD41XX_VBUS_TYPE_PHYPRV) {
		if (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY) {
			control_vcmpen_on();
			control_vbusin_thru();
		}
		else {
			control_vbusin_out();
		}
		ret = request_irq(IRQ_AVALID_RISE, det_vbuscid, IRQF_DISABLED,
				  "usb_mcd_cxd41xx_vbus_rise", NULL);
		if (ret < 0) {
			return ret;
		}
		ret = request_irq(IRQ_AVALID_FALL, det_vbuscid, IRQF_DISABLED,
				  "usb_mcd_cxd41xx_vbus_fall", NULL);
		if (ret < 0) {
			return ret;
		}
	}
#endif				/* !defined(CONFIG_ARCH_CXD4105) */

	/* CID */
	if (port->cid_type == _CXD41XX_CID_TYPE_GPIO) {
		gpio_mode mode;
		ret = gpio_register_bit(port->cid.prt, port->cid.bit);
		if (ret < 0) {
			return ret;
		}
		mode = GPIOMODE_INPUT;
#if defined(CONFIG_ARCH_CXD4105)
		mode |= GPIOMODE_PULLUP;
#endif				/* defined(CONFIG_ARCH_CXD4105) */
		ret = gpio_set_mode_bit(port->cid.prt, port->cid.bit, mode);
		if (ret < 0) {
			return ret;
		}
		ret = request_vbuscid_irq(&port->cid);
		if (ret < 0) {
			return ret;
		}
	}
	else if (port->cid_type == _CXD41XX_CID_TYPE_PHY) {
		control_iddig_thru();
		control_idpullup_on();
	}

	return ret;
}

static void stop_port(struct usb_otg_control *otg_control,
		      struct _cxd41xx_port *port)
{
	/* CID */
	if (port->cid_type == _CXD41XX_CID_TYPE_GPIO) {
		if (check_gpio_sys(port->cid.prt)) {
#if defined(CONFIG_ARCH_CXD4105)
			free_irq(IRQ_GPIOx(port->cid.bit), &port->cid);
#else
			free_irq(IRQ_GPIO_RISE(port->cid.bit), &port->cid);
			free_irq(IRQ_GPIO_FALL(port->cid.bit), &port->cid);
#endif				/* defined(CONFIG_ARCH_CXD4105) */
		}
		gpio_unregister_bit(port->cid.prt, port->cid.bit);
	}
	else if (port->cid_type == _CXD41XX_CID_TYPE_PHY) {
		control_idpullup_off();
	}

	/* VBUS */
	if (port->vbus_type == _CXD41XX_VBUS_TYPE_GPIO
	    || port->vbus_type == _CXD41XX_VBUS_TYPE_GPIOX) {
		if (check_gpio_sys(port->vbus.prt)) {
#if defined(CONFIG_ARCH_CXD4105)
			free_irq(IRQ_GPIOx(port->vbus.bit), &port->vbus);
#else
			free_irq(IRQ_GPIO_RISE(port->vbus.bit), &port->vbus);
			free_irq(IRQ_GPIO_FALL(port->vbus.bit), &port->vbus);
#endif				/* defined(CONFIG_ARCH_CXD4105) */
		}
		gpio_unregister_bit(port->vbus.prt, port->vbus.bit);
	}
#if defined(CONFIG_ARCH_CXD4105)
	else if (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY) {
		control_vcmpen_off();
		control_vbusin_off();
	}
#else
	else if (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY ||
		 port->vbus_type ==_CXD41XX_VBUS_TYPE_PHYPRV) {
		free_irq(IRQ_AVALID_RISE, NULL);
		free_irq(IRQ_AVALID_FALL, NULL);
		if (port->vbus_type == _CXD41XX_VBUS_TYPE_PHY) {
			control_vcmpen_off();
			control_vbusin_off();
		}
	}
#endif				/* !defined(CONFIG_ARCH_CXD4105) */
}

static int start_select_port(struct usb_otg_control *otg_control,
			     struct _cxd41xx_select_port *select_port, u32 bit)
{
	int ret = 0;
	gpio_mode mode;

	ret = gpio_register_bit(select_port->sel.prt, select_port->sel.bit);
	if (ret < 0) {
		return ret;
	}

	mode = bit? GPIOMODE_OUTPUT_H : GPIOMODE_OUTPUT_L;
#if defined(CONFIG_ARCH_CXD4105)
	mode |= GPIOMODE_PUPDOFF;
#endif				/* defined(CONFIG_ARCH_CXD4105) */
	ret = gpio_set_mode_bit(select_port->sel.prt, select_port->sel.bit, mode);

	return ret;
}

static void stop_select_port(struct usb_otg_control *otg_control,
			     struct _cxd41xx_select_port *select_port)
{
	gpio_unregister_bit(select_port->sel.prt, select_port->sel.bit);
}

#if defined(CONFIG_USB_MCD_HOST)

/* request_oc_irq - oc߽ */
inline static int request_oc_irq(struct _cxd41xx_gpio *gpio)
{
	int ret = 0;
	const char devname[] = "usb_mcd_cxd41xx_oc";

#if defined(CONFIG_ARCH_CXD4105)
	ret = request_irq(IRQ_GPIOx(gpio->bit), det_oc, IRQF_DISABLED, devname,
			  gpio);
	if (ret < 0) {
		return ret;
	}
	ret = gpio_set_intr_trig_bit(gpio->prt, gpio->bit, GPIO_TRG_LOW_EDGE);
	if (ret < 0) {
		return ret;
	}
	ret = gpio_ctrl_intr_en_bit(gpio->prt, gpio->bit, 1);
	if (ret < 0) {
		return ret;
	}
#else
	ret = request_irq(IRQ_GPIO_FALL(gpio->bit), det_oc, IRQF_DISABLED,
			  devname, gpio);
	if (ret < 0) {
		return ret;
	}
#endif				/* defined(CONFIG_ARCH_CXD4105) */

	return 0;
}

static int start_oc(struct usb_otg_control *otg_control,
		    struct _cxd41xx_port_control *port_control)
{
	int ret = 0;
	gpio_mode mode;

	/* СGPIO */
	ret = gpio_register_bit(port_control->oc.prt, port_control->oc.bit);
	if (ret < 0) {
		return ret;
	}

	mode = GPIOMODE_INPUT;
#if defined(CONFIG_ARCH_CXD4105)
	mode |= GPIOMODE_PULLUP;
#endif				/* defined(CONFIG_ARCH_CXD4105) */
	ret = gpio_set_mode_bit(port_control->oc.prt, port_control->oc.bit, mode);
	if (ret < 0) {
		return ret;
	}

	/* OverCurrentг */
	ret = request_oc_irq(&port_control->oc);
	if (ret < 0) {
		return ret;
	}

	return ret;
}

static void stop_oc(struct usb_otg_control *otg_control,
		    struct _cxd41xx_port_control *port_control)
{
#if defined(CONFIG_ARCH_CXD4105)
	free_irq(IRQ_GPIOx(port_control->oc.bit), &port_control->oc);
#else
	free_irq(IRQ_GPIO_FALL(port_control->oc.bit), &port_control->oc);
#endif				/* defined(CONFIG_ARCH_CXD4105) */
	gpio_unregister_bit(port_control->oc.prt, port_control->oc.bit);
}

#endif					/* defined(CONFIG_USB_MCD_HOST) */

static int start_control(struct usb_otg_control *otg_control,
			 struct mcd_otg_intdat *otgc_intdat,
			 struct _cxd41xx_port_control *port_control)
{
	u32 i;
	unsigned long flags;
	struct _cxd41xx_port *port;
	struct _cxd41xx_select_port *select_port;
	u32 current_select_pattern;
	u32 bit;
	int ret = 0;

	/* Ͻ */
	set_pparam(pparam);
	/* PHYåON */
	phy_clock_on();
	clock_off();
	if (!check_using_phy_vbus(otgc_intdat)) {
		/* PHYPRVVbus񤭤ʤ */
		control_vbusin_off();
	}
	if (!check_using_phy_cid(otgc_intdat)) {
		control_iddig_b();
		control_idpullup_off();
	}

	spin_lock_irqsave(&otgc_intdat->lock, flags);

	/* Port */
	i = 0;
	while (i < port_control->port_num) {

		port = &port_control->port[i];

		/* OTGȥPortå */
		ret = mcd_otg_control_attach_port(
			otg_control, i, 3, 3,
			check_cid_funcs[port->cid_type],
			check_vbus_funcs[port->vbus_type]);
		if (ret < 0) {
			err("mcd_otg_control_attach_port(%d) %d\n", i, ret);
			goto control_io_start_error;
		}

		ret = start_port(otg_control, port);
		if (ret < 0) {
			err("start_port(%d) %d\n", i, ret);
			goto control_io_start_error;
		}

		i++;
	}

	/* ݡȤѥ */
	current_select_pattern =
		port_control->port[port_control->current_port].
			select_pattern;

	/* GPIO */
	i = 0;
	while (i < port_control->select_port_num) {

		select_port = &port_control->select_port[i];
		bit = current_select_pattern >> i & 0x1;
		ret = start_select_port(otg_control, select_port, bit);
		if (ret < 0) {
			err("start_select_port(%d:%d) %d,%d\n",
			    select_port->sel.prt, select_port->sel.bit,
			    i, ret);
			goto control_io_start_error;
		}

		i++;
	}

#if defined(CONFIG_USB_MCD_HOST)
	if (check_gpio_sys(port_control->oc.prt)) {
		ret = start_oc(otg_control, port_control);
		if (ret < 0) {
			err("start_oc(%d:%d) %d\n", port_control->oc.prt,
			    port_control->oc.bit, ret);
			goto control_io_start_error;
		}

	}
	/* Vbusüһ */
	init_vbusout();
#endif				/* defined(CONFIG_USB_MCD_HOST) */

control_io_start_error:
	spin_unlock_irqrestore(&otgc_intdat->lock, flags);

	return ret;
}

static int stop_control(struct usb_otg_control *otg_control,
			 struct mcd_otg_intdat *otgc_intdat,
			 struct _cxd41xx_port_control *port_control)
{
	u32 i;
	unsigned long flags;
	struct _cxd41xx_port *port;
	struct _cxd41xx_select_port *select_port;
	int ret = 0;

	spin_lock_irqsave(&otgc_intdat->lock, flags);

	/* λ */
#if defined(CONFIG_USB_MCD_HOST)
	/* OverCurrent */
	if (check_gpio_sys(port_control->oc.prt)) {
		stop_oc(otg_control, port_control);
	}
	/* Vbusüһ */
	exit_vbusout();
#endif				/* defined(CONFIG_USB_MCD_HOST) */

	/* GPIO */
	i = port_control->select_port_num;
	while (i > 0) {
		i--;
		select_port = &port_control->select_port[i];
		stop_select_port(otg_control, select_port);
	}

	/* Port */
	i = port_control->port_num;
	while (i > 0) {
		i--;
		port = &port_control->port[i];
		stop_port(otg_control, port);

		/* OTGȥ餫Portǥå */
		mcd_otg_control_detach_port(otg_control, i);
	}

	clock_off();

	spin_unlock_irqrestore(&otgc_intdat->lock, flags);

	return ret;
}

static int start_gadget(struct mcd_otg_intdat *otgc_intdat)
{
	/* VBUS PHY̤ѻVBUS_ON */
	if (!check_using_phy_vbus(otgc_intdat)) {
		control_vbusin_on();
		control_vcmpen_off();
	}

	if (!check_using_phy_cid(otgc_intdat)) {
		control_iddig_b();
	}

	clock_on();

	return 0;
}

#if defined(CONFIG_USB_MCD_HOST)
static int start_host(struct mcd_otg_intdat *otgc_intdat)
{
	if (!check_using_phy_cid(otgc_intdat)) {
		control_iddig_a();
	}

	clock_on();

	return 0;
}

static int start_vbusout(struct mcd_otg_intdat *otgc_intdat)
{
	/* VBUS PHY̤ѻVBUS_ON */
	if (!check_using_phy_vbus(otgc_intdat)) {
		control_vbusin_on();
		control_vcmpen_off();
	}

	enable_vbusout();

	return 0;
}

static int stop_vbusout(struct mcd_otg_intdat *otgc_intdat)
{
	disable_vbusout();

	/* VBUS PHY̤ѻVBUS_OFF */
	if (!check_using_phy_vbus(otgc_intdat)) {
		control_vbusin_off();
		control_vcmpen_off();
	}

	return 0;
}
#endif				/* defined(CONFIG_USB_MCD_HOST) */

static int stop_device(struct mcd_otg_intdat *otgc_intdat)
{
	clock_off();

	if (!check_using_phy_cid(otgc_intdat)) {
		control_iddig_b();
	}

#if defined(CONFIG_USB_MCD_HOST)
	disable_vbusout();
#endif				/* defined(CONFIG_USB_MCD_HOST) */

	/* VBUS PHY̤ѻVBUS_OFF */
	if (!check_using_phy_vbus(otgc_intdat)) {
		control_vbusin_off();
		control_vcmpen_off();
	}

	return 0;
}

#if defined(CONFIG_USB_MCD_HOST)
static int wait_srp(struct mcd_otg_intdat *otgc_intdat)
{
	struct mcd_device* mdev = otgi_to_mdev(otgc_intdat);
	void *regs = otgi_regs(otgc_intdat);

	control_iddig_a();
	disable_vbusout();

	clock_off();
	clock_on();

	mcd_device_init(mdev);
	MCD_WRITE8(regs, MCD_O_HDRC_INTRUSBE, MCD_M_INTR_SESSREQ);

	control_vcmpen_on();
	control_vbusin_thru();

	return 0;
}
#endif				/* defined(CONFIG_USB_MCD_HOST) */

static int do_idle(struct mcd_otg_intdat *otgc_intdat)
{
	u32 pn;
	bool iddig;
	struct _cxd41xx_port_control *port_control;
	int ret = 0;

	port_control = &cxd41xx_port_control;
	pn = port_control->current_port;
	iddig = otgc_intdat->port[pn].last_cid;

#if defined(CONFIG_USB_MCD_HOST)
	if (iddig) {
		disable_vbusout();
	}
#endif				/* defined(CONFIG_USB_MCD_HOST) */

	if (!check_using_phy_cid(otgc_intdat)) {
		if (iddig) {
			control_iddig_b();
		}
		else {
			control_iddig_a();
		}
	}

	if (otgc_intdat->state == MCD_OTGC_S_AB_IDLE) {
		if (iddig) {
			ret = stop_device(otgc_intdat);
		}
#if defined(CONFIG_USB_MCD_HOST)
		else {
			ret = wait_srp(otgc_intdat);
		}
#endif				/* defined(CONFIG_USB_MCD_HOST) */
	}

	return ret;
}

static int init_srp(void)
{
	clock_on();
	control_vcmpen_on();
	control_vbusin_thru();

	return 0;
}

static int suspend(struct _cxd41xx_port_control *port_control)
{
	suspend_save(&port_control->save);

	return 0;
}

static int resume(struct _cxd41xx_port_control *port_control)
{
	/* PHYåON */
	phy_clock_on();

	resume_restore(&port_control->save, pparam);

	return 0;
}

static int reset_host(struct mcd_otg_intdat *otgc_intdat)
{
	struct mcd_device* mdev = otgi_to_mdev(otgc_intdat);
	void *regs = otgi_regs(otgc_intdat);
	struct usb_hcd *hcd = mdev->hcd;

	clock_off();
	clock_on();

	if (hcd) {
		if (hcd->driver->stop) {
			hcd->driver->stop(hcd);
		}
		if (hcd->driver->start) {
			hcd->driver->start(hcd);
		}
	}

	MCD_WRITE8(regs, MCD_O_HDRC_DEVCTL, MCD_M_DEVCTL_SESSION);

	return 0;
}

/*
 * ؿcontrol_io
 * usb_otg_control_start_control, usb_otg_control_start_control
 * ƤӽФ롣
 */
static int control_io(struct usb_otg_control *otg_control, u32 control)
{
	int ret = 0;
	struct mcd_otg_intdat *otgc_intdat;
	struct _cxd41xx_port_control *port_control;

	DBG(2, "<==\n");

	otgc_intdat = otgc_to_otgi(cxd41xx_otgc);
	port_control = &cxd41xx_port_control;

	switch (control) {
	case USB_OTG_IO_START_CONTROL:
		ret = start_control(otg_control, otgc_intdat, port_control);
		break;
	case USB_OTG_IO_STOP_CONTROL:
	case USB_OTG_IO_SHUTDOWN:
		ret = stop_control(otg_control, otgc_intdat, port_control);
		break;
	case USB_OTG_IO_START_GADGET:
		ret = start_gadget(otgc_intdat);
		break;
#if defined(CONFIG_USB_MCD_HOST)
	case USB_OTG_IO_START_HOST:
		ret = start_host(otgc_intdat);
		break;
#endif				/* defined(CONFIG_USB_MCD_HOST) */
	case USB_OTG_IO_STOP_GADGET:
	case USB_OTG_IO_STOP_HOST:
	case USB_OTG_IO_STOP_RCHOST:
	case USB_OTG_IO_STOP_RCGADGET:
	case USB_OTG_IO_CHANGE_CID:
	case USB_OTG_IO_FINISH_SRP:
		ret = do_idle(otgc_intdat);
		break;
	case USB_OTG_IO_INIT_SRP:
		ret = init_srp();
		break;
#if defined(CONFIG_USB_MCD_HOST)
	case USB_OTG_IO_START_VBUSOUT:
		ret = start_vbusout(otgc_intdat);
		break;
	case USB_OTG_IO_STOP_VBUSOUT:
		ret = stop_vbusout(otgc_intdat);
		break;
#endif				/* defined(CONFIG_USB_MCD_HOST) */
	case USB_OTG_IO_SUSPEND:
		ret = suspend(port_control);
		break;
	case USB_OTG_IO_RESUME:
		ret = resume(port_control);
		break;
	case USB_OTG_IO_DISCONN:
		ret = reset_host(otgc_intdat);
		break;
	default:
		break;
	}

	DBG(2, "==>\n");
	return ret;
}

/*
 * ؿget_port_info
 */
static int get_port_info(struct usb_otg_control *otg_control,
			 struct usb_otg_control_port_info *info)
{
	struct _cxd41xx_port_control *port_control = &cxd41xx_port_control;

	info->nr = port_control->port_num;
	info->current_port = port_control->current_port;

	return 0;
}

/*
 * ؿselect_port
 */
static int select_port(struct usb_otg_control *otg_control, u32 pn)
{
	int ret;
	u32 i;
	struct mcd_otg_intdat *otgc_intdat;
	struct _cxd41xx_port_control *port_control = &cxd41xx_port_control;
	struct _cxd41xx_select_port *select_port;
	u32 current_select_pattern;
	gpio_bit bit;

	if (port_control->port_num <= pn) {
		return -EIO;
	}

	otgc_intdat = otgc_to_otgi(cxd41xx_otgc);

	/* ݡ */
	port_control->current_port = pn;

	/* ݡȤѥ */
	current_select_pattern =
	    port_control->port[port_control->current_port].select_pattern;

	/* GPIO */
	for (i = 0; i < port_control->select_port_num; i++) {

		select_port = &port_control->select_port[i];

		bit = current_select_pattern >> i & 0x1;
		ret = gpio_set_data_bit(select_port->sel.prt,
					select_port->sel.bit, bit);
	}

	return do_idle(otgc_intdat);
}

/*
 * ؿcontrol_vbus
 */
static int control_vbus(struct usb_otg_control *otg_control, u32 control)
{
	DBG(2, "<==\n");

	if (control == USB_OTG_ON) {
		control_vbusin_on();
	} else if (control == USB_OTG_OFF) {
		control_vbusin_off();
	} else if (control == USB_OTG_THRU) {
		control_vbusin_thru();
	} else {
		return -EINVAL;
	}

	DBG(2, "==>\n");
	return 0;
}

/*
 * ؿcontrol_test
 * usb_otg_control_set_test_modeˡ쥸˸ƤӽФ롣
 */
static int control_test(struct usb_otg_control *otg_control,
			enum usb_otg_control_test_mode test_mode)
{
	DBG(2, "<==\n");

	switch (test_mode) {
#if defined(CONFIG_USB_MCD_FORCE_HOST)
	case USB_OTG_TEST_MODE_H_AS:
		msleep(1);
		control_iddig_a();
		control_vbusin_on();
		DBG(2, "set test mode : H_AS\n");
		break;
#endif
	case USB_OTG_TEST_MODE_NORMAL:
#if defined(CONFIG_USB_MCD_HOST)
		control_iddig_thru();
		control_vbusin_thru();
#else
		control_iddig_b();
		control_vbusin_on();
#endif				/* !defined(CONFIG_USB_MCD_HOST) */
		DBG(2, "set test mode : NORMAL\n");
		break;
	case USB_OTG_TEST_MODE_G_FORCE_PACKET:
		control_vbusin_on();
		DBG(2, "set test mode : FORCE_PACKET\n");
		break;
	default:
		break;
	}

	DBG(2, "==>\n");
	return 0;
}

/*
 * mcd_otg_intdatڥ졼ؿ
 */
struct mcd_otg_intdat_ops cxd41xx_otgc_ops = {
	.get_port_info = get_port_info,
	.select_port = select_port,
	.check_suspend = check_suspend,
	.control_io = control_io,
	.control_vbus = control_vbus,
	.control_test = control_test,
};

/**************************** procfs functions *****************************/

/*
 * ؿpparam_parse
 */
static char *pparam_parse(char *str, u8 *value)
{
	const char hexchars[] = "0123456789abcdef";
	u8 tmp;

	*value = 0;

	do {
		for (tmp = 0; hexchars[tmp] != '\0'; tmp++) {
			if (*str == hexchars[tmp]) {
				*value <<= 4;
				*value += tmp;
				break;
			}
		}
		if (hexchars[tmp] == '\0') {
			return NULL;
		}
		str++;
	} while (*str != '\0' && *str != ',');

	if (*str == ',') {
		str++;
	}

	return str;
}

/*
 * ؿpparam_proc_write
 */
static int pparam_proc_write(struct file *file, const char *buffer,
			     unsigned long count, void *data)
{
	int res;
	u32 i;
	char param[PPARAM_WRITE_BUFSIZE+1];
	u8 tmp[BOARD_PPARAM_NUM];
	char *pstr;

	count = min(count, (unsigned long)PPARAM_WRITE_BUFSIZE);

	/* User֤饳ԡ */
	if (copy_from_user(param, buffer, count)) {
		res = -EFAULT;
		goto exit;
	}
	param[count] = '\0';
	res = count;

	DBG(2, "count: %ld (%s)\n", count, param);

	/* setޥɤǧ */
	if (strncmp(param, "set ", 4) != 0) {
		DBG(2, "parse error\n");
		goto exit;
	}

	pstr = &param[4];

	for (i = 0; i < BOARD_PPARAM_NUM; i++) {
		pstr = pparam_parse(pstr, &tmp[i]);
		if ((pstr == NULL || *pstr == '\0') &&
		    i != (BOARD_PPARAM_NUM - 1)) {
			err("parse error. default param will be used.\n");	
			goto exit;
		}
		DBG(2, "set pparam[%d]: 0x%04x\n", i, tmp[i]);
	}

	memcpy(pparam, tmp, sizeof tmp);

	DBG(2, "copy tmp -> pparam count: %d\n", sizeof tmp);

      exit:
	return res;
}

/*
 * ؿset_port_desc_token
 * ʸ':'Ƕڤꡢʸ֤
 */
static char *set_port_desc_token(char *pos)
{
	char *next;

	next = strchr(pos, ':');

	if (!next) {
		return NULL;
	}

	*next = '\0';
	next++;

	return next;
}

/*
 * ؿparse_port_desc
 * Ϥ줿ǥץϤport_controlꤹ롣
 */
static int
parse_port_desc(struct _cxd41xx_port_control *port_control, char *desc)
{
	u32 i;
	char *pos;
	char *next;
	int retval;
	u32 cid_type;
	u32 cid_prt;
	u32 cid_bit;
	u32 vbus_type;
	u32 vbus_prt;
	u32 vbus_bit;
	u32 sel_prt;
	u32 sel_bit;
	u32 oc_prt;
	u32 oc_bit;
	struct _cxd41xx_port *port;
	struct _cxd41xx_select_port *select_port;

	/* port info */
	next = set_port_desc_token(desc);
	if (!next) {
		err("lack of colon(Port num)\n");
		return -EFAULT;
	}

	retval = sscanf(desc, " %d", &port_control->port_num);
	if (retval < 1) {
		err("Invalid parameter(Port num)\n");
		return -EINVAL;
	}
	if (MCD_MAX_PORT_NUM < port_control->port_num) {
		err("Max over num(Port num)\n");
		return -EINVAL;
	}

	for (i = 0; i < port_control->port_num; i++) {
		port = &port_control->port[i];

		pos = next;
		next = set_port_desc_token(pos);
		if (!next) {
			err("lack of colon(Port%d)\n", i);
			return -EFAULT;
		}

		retval = sscanf(pos, " %d , %d , %d . %d , %d . %d , %x ",
				&cid_type, &vbus_type,
				&cid_prt, &cid_bit, &vbus_prt, &vbus_bit,
				&port->select_pattern);
		port->cid_type = cid_type;
		port->vbus_type = vbus_type;
		port->cid.prt = cid_prt;
		port->cid.bit = cid_bit;
		port->vbus.prt = vbus_prt;
		port->vbus.bit = vbus_bit;
		if (retval < 5) {
			err("Invalid parameter(Port%d), %d\n", i, retval);
			return -EINVAL;
		}
		/* CID, Vbus typeΥå */
		if (ARRAY_SIZE(check_cid_funcs) <= port->cid_type) {
			err("CID type is invalid(Port%d) %d\n", i,
			    port->cid_type);
			return -EINVAL;
		}
		if (ARRAY_SIZE(check_vbus_funcs) <= port->vbus_type) {
			err("Vbus type is invalid(Port%d) %d\n", i,
			    port->vbus_type);
			return -EINVAL;
		}
	}

	/* select GPIO info */
	pos = next;
	next = set_port_desc_token(pos);
	if (!next) {
		err("lack of colon(Select port num)\n");
		return -EFAULT;
	}

	retval = sscanf(pos, " %d ", &port_control->select_port_num);
	if (retval < 1) {
		err("Invalid parameter(Select port num) %d, d\n", retval);
		return -EINVAL;
	}
	if (MCD_MAX_SELECT_PORT_NUM < port_control->select_port_num) {
		err("Max over num(Select port num)\n");
		return -EINVAL;
	}

	for (i = 0; i < port_control->select_port_num; i++) {
		select_port = &port_control->select_port[i];

		pos = next;
		next = set_port_desc_token(pos);
		if (!next) {
			err("lack of colon(Select port%d)\n", i);
			return -EFAULT;
		}

		retval = sscanf(pos, " %d . %d ", &sel_prt, &sel_bit);
		select_port->sel.prt = sel_prt;
		select_port->sel.bit = sel_bit;
		if (retval < 1) {
			err("Invalid parameter(Select port%d), %d\n", i,
			    retval);
			return -EINVAL;
		}
	}

	/* oc GPIO info */
	pos = next;
	next = set_port_desc_token(pos);
	if (!next) {
		err("lack of colon(Oc)\n");
		return -EFAULT;
	}

	retval = sscanf(pos, " %d . %d ", &oc_prt, &oc_bit);
	port_control->oc.prt = oc_prt;
	port_control->oc.bit = oc_bit;
	if (retval < 1) {
		err("Invalid parameter(Oc) %d, d\n", retval);
		return -EINVAL;
	}

	return 0;
}

/*
 * ؿread_port_desc
 * port_controlɽ롣
 */
static int
read_port_desc(char *page,
	       char **start, off_t off, int count, int *eof, void *data)
{
	u32 i;
	struct usb_otg_control *otg_control;
	struct _cxd41xx_port_control *port_control;
	struct _cxd41xx_port *port;
	struct _cxd41xx_select_port *select_port;
	char *pos;
	s32 len;
	char desc[MCD_PORT_DESC_SIZE(MCD_MAX_PORT_NUM,
				     MCD_MAX_SELECT_PORT_NUM)];
	s32 sz = sizeof desc;

	otg_control = cxd41xx_otgc;
	port_control = &cxd41xx_port_control;

	pos = desc;

	/* port info */
	len = snprintf(pos, sz, "%d:", port_control->port_num);
	if (len < 0)
		goto read_error;
	pos += len;
	sz -= len;

	for (i = 0; i < port_control->port_num; i++) {
		port = &port_control->port[i];
		if (sz <= 0)
			goto buf_over;
		len = snprintf(pos, sz, "%d,%d,%d.%d,%d.%d,0x%x:",
			       port->cid_type, port->vbus_type,
			       port->cid.prt, port->cid.bit,
			       port->vbus.prt, port->vbus.bit,
			       port->select_pattern);
		if (len < 0)
			goto read_error;
		pos += len;
		sz -= len;
	}

	/* select GPIO info */
	if (sz <= 0)
		goto buf_over;
	len = snprintf(pos, sz, "%d:", port_control->select_port_num);
	if (len < 0)
		goto read_error;
	pos += len;
	sz -= len;

	for (i = 0; i < port_control->select_port_num; i++) {
		select_port = &port_control->select_port[i];
		if (sz <= 0)
			goto buf_over;
		len = snprintf(pos, sz, "%d.%d:", select_port->sel.prt,
			       select_port->sel.bit);
		if (len < 0)
			goto read_error;
		pos += len;
		sz -= len;
	}

	/* oc GPIO info */
	if (sz <= 0)
		goto buf_over;
	len = snprintf(pos, sz, "%d.%d:\n", port_control->oc.prt,
		       port_control->oc.bit);
	if (len < 0)
		goto read_error;
	pos += len;
	sz -= len;

	if (sz > 0)
		*pos = '\0';

	len = min(count, pos - desc);
	memcpy(page, desc, len);

	*eof = 1;
	return len;

buf_over:
	len = -EOVERFLOW;
read_error:
	return len;
}

/*
 * ؿwrite_port_desc
 * IDLEֻͿ줿ʸport_controlꤹ롣
 */
static int
write_port_desc(struct file *file,
		const char *buffer, unsigned long count, void *data)
{
	struct usb_otg_control *otg_control;
	struct mcd_otg_intdat *otgc_intdat;
	struct _cxd41xx_port_control *port_control;
	unsigned long flags;
	char desc[MCD_PORT_DESC_SIZE(MCD_MAX_PORT_NUM,
				     MCD_MAX_SELECT_PORT_NUM)];
	int retval;
        unsigned long sts;

	if (count >= sizeof(desc)) {
		return -EOVERFLOW;
	}

	otg_control = cxd41xx_otgc;
	otgc_intdat = otgc_to_otgi(cxd41xx_otgc);
	port_control = &cxd41xx_port_control;

        sts = copy_from_user(desc, buffer, count);
	desc[count] = '\0';

	spin_lock_irqsave(&otgc_intdat->lock, flags);

	if (otgc_intdat->state != MCD_OTGC_S_STOP) {
		retval = -EBUSY;
	} else if (count < 15 || desc[0] == '0') {
		port_control->port_num = 0;
		port_control->select_port_num = 0;
		retval = count;
	} else {
		retval = parse_port_desc(port_control, desc);
		if (retval == 0) {
			retval = count;
		}
		/* ° */
		otgc_intdat->attributes = 0;
		set_attr_all_int_port(otgc_intdat, port_control);
		set_attr_using_phy_vbus(otgc_intdat, port_control);
		set_attr_using_phy_cid(otgc_intdat, port_control);
	}

	spin_unlock_irqrestore(&otgc_intdat->lock, flags);

	return retval;
}

/**************************** global functions *****************************/

#if !defined(MCD_DEFAULT_TIMER)

#define TIMER_CTL		(TMCK_DIV1 | TMCS_FREERUN | TMST)

/* timer */
struct _cxd41xx_timer {
	struct mcd_device *mdev;
	void (*func)(struct mcd_device *, void *);
	void *data;
	u8 timer;
};

/*
 * IRQ handler for the timer
 */
static irqreturn_t timer_irq(int irq, void *ptr)
{
	struct _cxd41xx_timer *mdev_timer;

	mdev_timer = (struct _cxd41xx_timer *)ptr;

	/* Clear */
	writel(TIMER_CTL, VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCTL);
	writel(TMINTCLR,
	       VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCLR);

	if (mdev_timer->func) {
		mdev_timer->func(mdev_timer->mdev, mdev_timer->data);
	}
	mdev_timer->func = NULL;
	mdev_timer->data = NULL;

	return IRQ_HANDLED;
}

int mcd_board_init_timer(struct mcd_device *mdev)
{
	int ret;
	struct _cxd41xx_timer *mdev_timer;

	mdev->timer.data = (void *)kzalloc(sizeof(*mdev_timer), GFP_KERNEL);
	if (!mdev->timer.data) {
		return -ENOMEM;
	}

	mdev_timer = (struct _cxd41xx_timer *)mdev->timer.data;
	mdev_timer->timer = TIMER_FOR_USB;

	/* Clear */
	writel(0, VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCTL);
	writel(TMCLR|TMINTCLR,
	       VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCLR);

#if defined(CONFIG_ARCH_CXD4105)
	ret = request_irq(IRQ_TIMERx(mdev_timer->timer), timer_irq,
			   IRQF_DISABLED, "usb_mcd_cxd41xx_timer", mdev_timer);
#else
	ret = request_irq(IRQ_TIMER(mdev_timer->timer), timer_irq,
			   IRQF_DISABLED, "usb_mcd_cxd41xx_timer", mdev_timer);
#endif				/* defined(CONFIG_ARCH_CXD4105) */
	if (ret < 0) {
		return ret;
	}

	/* timer start */
	writel(TIMER_CTL, VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCTL);

	return 0;
}

int mcd_board_exit_timer(struct mcd_device *mdev)
{
	unsigned long flags;
	struct _cxd41xx_timer *mdev_timer;

	mdev_timer = (struct _cxd41xx_timer *)mdev->timer.data;

	spin_lock_irqsave(&mdev->lock, flags);

	/* Clear */
	writel(0, VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCTL);
	writel(TMCLR|TMINTCLR,
	       VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCLR);

	mdev_timer->func = NULL;
	mdev_timer->data = NULL;

	spin_unlock_irqrestore(&mdev->lock, flags);

	/* release resources */
#if defined(CONFIG_ARCH_CXD4105)
	free_irq(IRQ_TIMERx(mdev_timer->timer), mdev_timer);
#else
	free_irq(IRQ_TIMER(mdev_timer->timer), mdev_timer);
#endif				/* defined(CONFIG_ARCH_CXD4105) */
	kfree(mdev_timer);
	mdev->timer.data = NULL;

	return 0;
}

/* 1[mcd_time_t] is 100/128[us] */
/* 1[mcd_time_t] is 1/4[us] at CXD4115 */
u32 mcd_board_time2usec(mcd_time_t time)
{
#if defined(CONFIG_ARCH_CXD4105)
	return (u32)(time * 100 / 128);
#else
	return (u32)(time / 4);
#endif				/* defined(CONFIG_ARCH_CXD4105) */
}
EXPORT_SYMBOL(mcd_board_time2usec);

mcd_time_t mcd_board_usec2time(u32 usec)
{
#if defined(CONFIG_ARCH_CXD4105)
	return (mcd_time_t)(usec * 128 / 100);
#else
	return (mcd_time_t)(usec * 4);
#endif				/* defined(CONFIG_ARCH_CXD4105) */
}
EXPORT_SYMBOL(mcd_board_usec2time);

mcd_time_t mcd_board_get_time(struct mcd_device *mdev)
{
	u32 now;
	struct _cxd41xx_timer *mdev_timer;

	mdev_timer = (struct _cxd41xx_timer *)mdev->timer.data;
	now = readl(VA_TIMER(mdev_timer->timer) + CXD4105_TIMERREAD);
	return (mcd_time_t)now;
}
EXPORT_SYMBOL(mcd_board_get_time);

int mcd_board_start_timer(struct mcd_device *mdev, mcd_time_t expire,
			  void (*func)(struct mcd_device *, void *),
			  void *data)
{
	unsigned long flags;
	struct _cxd41xx_timer *mdev_timer;
	u32 now;
	u32 cnl = TIMER_CTL;

	if (!mdev || !expire || !func) {
		return -EINVAL;
	}

	mdev_timer = (struct _cxd41xx_timer *)mdev->timer.data;
	if (mdev_timer->func) {
		return -EBUSY;
	}

	/* Enable interrupt and start the system timer */
	if (func) {
		cnl |= TMINT;
	}

	spin_lock_irqsave(&mdev->lock, flags);

	/* Setup the system timer compare register */
	now = readl(VA_TIMER(mdev_timer->timer) + CXD4105_TIMERREAD);
	writel(now + (u32)expire,
	       VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCMP);

	mdev_timer->func = func;
	mdev_timer->data = data;

	writel(cnl, VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCTL);

	spin_unlock_irqrestore(&mdev->lock, flags);

	return 0;
}
EXPORT_SYMBOL(mcd_board_start_timer);

void mcd_board_stop_timer(struct mcd_device *mdev, void *data)
{
	unsigned long flags;
	struct _cxd41xx_timer *mdev_timer;

	mdev_timer = (struct _cxd41xx_timer *)mdev->timer.data;

	spin_lock_irqsave(&mdev->lock, flags);

	mdev_timer->func = NULL;
	mdev_timer->data = NULL;

	/* Clear */
	writel(TIMER_CTL, VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCTL);
	writel(TMINTCLR,
	       VA_TIMER(mdev_timer->timer) + CXD4105_TIMERCLR);

	spin_unlock_irqrestore(&mdev->lock, flags);
}
EXPORT_SYMBOL(mcd_board_stop_timer);

#endif				/* !defined(MCD_DEFAULT_TIMER) */

unsigned long mcd_board_timeout_phy(void)
{
	return TIMEOUT_FOR_PHY_LOCK;
}
EXPORT_SYMBOL(mcd_board_timeout_phy);

/* phy_reset phyꥻå */
void mcd_board_phy_reset(void)
{
	phy_reset();
}

/* phy SuspendǽOFF */
void mcd_board_control_suspend_off(void)
{
	control_suspend_off();
}
EXPORT_SYMBOL(mcd_board_control_suspend_off);

/* phy SuspendǽON */
void mcd_board_control_suspend_on(void)
{
	control_suspend_on();
}
EXPORT_SYMBOL(mcd_board_control_suspend_on);

/* linestateå */
void mcd_board_linestate_set(void)
{
	control_linestate_set();
}
EXPORT_SYMBOL(mcd_board_linestate_set);

/* max port number μ */
u32 mcd_board_get_max_port_num(void)
{
	return (u32)MCD_MAX_PORT_NUM;
}

/*
 * ؿmcd_board_dump
 * boardϢǥХå
 */
void mcd_board_dump(struct mcd_device* mdev)
{
	dump_misc();
}

#if defined(CONFIG_ARCH_CXD4105)
/*
 * ؿmcd_cid_pullup
 * CID pullup on/off
 */
int mcd_board_cid_pullup(int on)
{
	int i, ret = 0;
	struct _cxd41xx_port_control *port_control;
	struct _cxd41xx_port *port;

	port_control = &cxd41xx_port_control;

	/* Port */
	i = 0;
	while (i < port_control->port_num) {
		port = &port_control->port[i];

		/* CID */
		if (port->cid_type == _CXD41XX_CID_TYPE_GPIO) {
			gpio_mode mode;

			mode = GPIOMODE_INPUT;
			if (on == 0)
				mode |= GPIOMODE_PUPDOFF;
			else
				mode |= GPIOMODE_PULLUP;
			ret = gpio_set_mode_bit(port->cid.prt, port->cid.bit, mode);
		}
		i++;
	}

	return ret;
}
#endif			/* defined(CONFIG_ARCH_CXD4105) */

/*
 * ؿmcd_board_map
 * GPIO桼ν桼ؿ
 */

int mcd_board_map(struct mcd_device* mdev)
{
	struct proc_dir_entry *proc_entry;
	int retval;

	DBG(2, "<==\n");

	/* OTGȥ饪֥Ȥ */
	cxd41xx_otgc = &mdev->otgi.otgc;
	retval = mcd_otg_control_create_object(mdev, 10, &cxd41xx_otgc_ops,
					       "musbhdrc0");
	cxd41xx_port_control.current_port = 0;

	/* pparamprocե */
	proc_entry = create_proc_entry(PPARAM_PROCFILE,
				       S_IFREG | S_IRUGO | S_IWUSR, NULL);
	if (proc_entry) {
		/* set to struct proc_entry */
		proc_entry->data = NULL;
		proc_entry->owner = THIS_MODULE;
		proc_entry->write_proc = pparam_proc_write;
	}

	/* Port descriptorprocե */
	proc_entry = create_proc_entry(PORT_DESC_PROCFILE,
				       S_IFREG | S_IRUGO | S_IWUSR, NULL);
	if (proc_entry) {
		/* set to struct proc_entry */
		proc_entry->data = NULL;
		proc_entry->owner = THIS_MODULE;
		proc_entry->write_proc = write_port_desc;
		proc_entry->read_proc = read_port_desc;
	}

	/* BOARD */
	clock_off();
	cxd41xx_init(pparam);

	DBG(2, "==>\n");

	return 0;
}

/*
 * ؿmcd_board_unmap
 * GPIO桼νλ桼ؿ
 */
int mcd_board_unmap(struct mcd_device* mdev)
{
	DBG(2, "<==\n");
	/* GPIOλ */
	cxd41xx_exit();

	/* pparamprocե */
	remove_proc_entry(PPARAM_PROCFILE, NULL);

	/* Port descriptoryprocե */
	remove_proc_entry(PORT_DESC_PROCFILE, NULL);

	/* objectõ */
	mcd_otg_control_destroy_object(mdev);

	DBG(2, "==>\n");
	return 0;
}
