/*--------------------------------------------------------------------------*
 * drivers/usb/mcd/usb_mcd_debug.h
 * 
 * USB controller driver
 * 
 * Copyright 2005,2006 Sony Corporation
 * 
 * This code is based on debug.h,
 * Copyright 2005 Mentor Graphics Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 *
 * The Inventra Controller Driver for Linux is free software; you
 * can redistribute it and/or modify it under the terms of the GNU
 * General Public License version 2 as published by the Free Software
 * Foundation.
 *
 * The Inventra Controller Driver for Linux is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with The Inventra Controller Driver for Linux ; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 *
 * ANY DOWNLOAD, USE, REPRODUCTION, MODIFICATION OR DISTRIBUTION
 * OF THIS DRIVER INDICATES YOUR COMPLETE AND UNCONDITIONAL ACCEPTANCE
 * OF THOSE TERMS.THIS DRIVER IS PROVIDED "AS IS" AND MENTOR GRAPHICS
 * MAKES NO WARRANTIES, EXPRESS OR IMPLIED, RELATED TO THIS DRIVER.
 * MENTOR GRAPHICS SPECIFICALLY DISCLAIMS ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY; FITNESS FOR A PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT.  MENTOR GRAPHICS DOES NOT PROVIDE SUPPORT
 * SERVICES OR UPDATES FOR THIS DRIVER, EVEN IF YOU ARE A MENTOR
 * GRAPHICS SUPPORT CUSTOMER.
 *---------------------------------------------------------------------------*/

#ifndef _USB_MCD_DEBUG_H_
#define _USB_MCD_DEBUG_H_

/*
 * Linux HCD (Host Controller Driver) for HDRC and/or MHDRC.
 * Debug support routines
 *
 * $Revision: 1.6 $
 */

struct usb_ep;
struct list_head;
struct usb_request;
struct usb_ctrlrequest;

extern int mcd_debug_level;

extern void dump_urb(void *urb);

extern char *decode_csr0(uint16_t csr0);
extern char *decode_txcsr(uint16_t txcsr);
extern char *decode_devctl(uint16_t devclt);
extern char *decode_ep0stage(uint8_t stage);

extern char *dump_node(struct list_head *node);

extern char *decode_usb_ctrlrequest(const struct usb_ctrlrequest
				    *pControlRequest);

extern char *decode_request(struct usb_ctrlrequest *);

#ifndef MCD_DEBUG
#define MCD_DEBUG 0
#endif

#if MCD_DEBUG > 0

#define STATIC

#define _if_dbg_level(level) if ( (level>=-1 && mcd_debug_level>=level) || mcd_debug_level==level )
#define xprintk(level, facility, format, args...) \
        do { _if_dbg_level(level) { \
                printk(facility "%s %d: " format , \
                       __FUNCTION__, __LINE__ , ## args); \
        } } while (0)

#define DBG(level,fmt,args...) xprintk(level,KERN_INFO,fmt, ## args)
#define WARN(fmt, args...) xprintk(-1,KERN_WARNING,fmt, ## args)
#define INFO(fmt,args...) xprintk(-1,KERN_INFO,fmt, ## args)
#define ERR(fmt,args...) xprintk(-1,KERN_INFO,fmt, ## args)

#define DUMP_STRUCT(_name, _ptr) dump_struct("_ptr", _name, _ptr)
#define DEBUG_CODE(level, code)	\
        do { if ( (level>=-1 && mcd_debug_level>level) || \
                  mcd_debug_level==level) { code \
                  } \
        } while (0)
#define TRACE(n) DEBUG_CODE(n, printk(KERN_INFO "%s:%s:%d: trace\n", \
	__FILE__, __FUNCTION__, __LINE__); )
#define TRACE(n) DEBUG_CODE(n, printk(KERN_INFO "%s:%s:%d: trace\n", \
	__FILE__, __FUNCTION__, __LINE__); )

/* #define ASSERT_SPINLOCK_LOCKED(_x) do { if (!spin_is_locked(_x)) \
	ERR("@pre clause failed, _x must be locked\n"); } while (0) 
#define ASSERT_SPINLOCK_UNLOCKED(_x) do { if (spin_is_locked(_x)) \
	ERR("@pre clause failed, _x must be unlocked\n"); } while (0) */

#define ASSERT_SPINLOCK_LOCKED(_x)
#define ASSERT_SPINLOCK_UNLOCKED(_x)
#define mcd_debug_set_level(_x)	mcd_debug_level=(_x)
#define mcd_debug_get_level()	(mcd_debug_level)

/* debug no defined */
#else

#define ASSERT_SPINLOCK_LOCKED(_x)
#define ASSERT_SPINLOCK_UNLOCKED(_x)
#define mcd_debug_set_level(_x)
#define mcd_debug_get_level()	0

#define STATIC static
#define DEBUG_CODE(x, y)	do {}  while (0)
#define TRACE(n)		do {}  while (0)
#define DBG(fmt,args...)	do {}  while (0)
#define WARN(fmt,args...) 	do {}  while (0)
#define INFO(fmt,args...) 	do {}  while (0)
#define ERR(fmt,args...)  	do {}  while (0)
#define PARANOID( x )		do {}  while (0)

#endif

#if defined(MCD_LOCK_DEBUG)
extern unsigned int mcd_debug_lock_irq_count;
extern unsigned int mcd_debug_lock_count;
#define PRLOCK() \
        printk(KERN_INFO " PRLOCK %s %d (%d:%d)\n", \
               __FUNCTION__, __LINE__, \
               mcd_debug_lock_count, mcd_debug_lock_irq_count)
#ifdef spin_lock_irqsave
#undef spin_lock_irqsave
#endif
#define spin_lock_irqsave(LOCK,FLAGS) \
  {local_irq_save(FLAGS); mcd_debug_lock_irq_count++; PRLOCK();}
#ifdef spin_unlock_irqrestore
#undef spin_unlock_irqrestore
#endif
#define spin_unlock_irqrestore(LOCK,FLAGS) \
  {local_irq_restore(FLAGS); mcd_debug_lock_irq_count--; PRLOCK();}
#ifdef spin_lock
#undef spin_lock
#endif
#define spin_lock(LOCK) {mcd_debug_lock_count++; PRLOCK();}
#ifdef spin_unlock
#undef spin_unlock
#endif
#define spin_unlock(LOCK) {mcd_debug_lock_count--; PRLOCK();}
#else
#define PRLOCK()
#endif

#define HDBG(format, arg...)	INFO(format, arg)

#endif				//  _USB_MCD_DEBUG_H_
