/*-----------------------------------------------------------------------------
 * drivers/usb/mcd/usb_mcd_host.c
 * 
 * USB controller driver
 * 
 * Copyright 2009 Sony Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _USB_MCD_GADGET_H_
#define _USB_MCD_GADGET_H_

#define MCD_DMA
#define MCD_LOCAL_PAD	0xa5deadfe
#define MCD_PAD_FRONT	0xa5deadfe
#define MCD_PAD_BACK	0xabadcafe

#define MCD_END0_STAGE_SETUP 		0x0
#define MCD_END0_STAGE_TX		0x2
#define MCD_END0_STAGE_RX		0x4
#define MCD_END0_STAGE_STATUSIN		0x8
#define MCD_END0_STAGE_STATUSOUT        0xf
#define MCD_END0_STAGE_STALL_BIT	0x10

/* EP0 IN status*/
#define MCD_EP0_COMP_END	0
#define MCD_EP0_COMP_NON	1

#define MCD_TEST_PACKET_SIZE	53
#define MCD_FIFO_MAXSIZE	0xFFFF
#define MCD_MAX_END0_PACKET	((uint16_t)MCD_END0_FIFOSIZE)
#define HCD_BUFFER_POOLS	4	//  Buffer_alloc

/* commodity  */
#define KMALLOC(a,b,c)	{ lock_kernel(); a=kmalloc(b,c); unlock_kernel(); }
#define KFREE(p)	{ lock_kernel(); kfree(p); unlock_kernel(); }
#define EP_NUMBER(_ep)		(((struct mcd_gep*)(_ep))->bEndNumber )
#define EP_IS_IDLE(_ep)		(list_empty(&((struct mcd_gep*)(_ep))->req_list))
#define EP_SPIN_LOCK(_ep)	spin_lock( &((struct mcd_gep*)(_ep))->Lock)
#define EP_SPIN_UNLOCK(_ep)	spin_unlock( &((struct mcd_gep*)(_ep))->Lock)
#define EP_SPIN_LOCK_IRQSAVE(_ep, _flags) \
	spin_lock_irqsave(&((struct mcd_gep*)(_ep))->Lock, _flags )
#define EP_SPIN_UNLOCK_IRQRESTORE(_ep, _flags) \
	spin_unlock_irqrestore(&((struct mcd_gep*)(_ep))->Lock, _flags)

/** Linux version
 */

#define GADGET_SET_IS_DUALSPEED(_g, _v)	do {(_g)->is_dualspeed = _v;} while(0)
#define GADGET_SET_IS_OTG(_g, _v)	do {(_g)->is_otg = _v;} while(0)
#define GADGET_SET_B_HNP_ENABLE(_g, _v)	do {(_g)->b_hnp_enable = _v;} while(0)
#define GADGET_SET_A_HNP_SUPPORT(_g, _v) do {(_g)->a_hnp_support = _v;} while(0)
#define GADGET_SET_A_ALT_HNP_SUPPORT(_g, _v) \
	do {(_g)->a_alt_hnp_support = _v;} while(0)

/* endpoint 0 buffer. When used with the standard ReadUSBHeader
 * routines, struct usb_ctrlrequest.
 */
struct mcd_end0_buffer {
	uint8_t aEnd0Data[MCD_MAX_END0_PACKET];
	uint16_t wEnd0Count;	/* bytes to go */
};

struct mcd_gep {
	struct usb_ep end_point;
#if defined(MCD_DEBUG)
	uint32_t wPadFront;
#endif
	spinlock_t Lock;
	struct list_head req_list;
	uint8_t bTrafficType;
	unsigned int dwRequestSize;
	uint16_t wPacketSize;
	uint8_t bIsTx;
	uint8_t bIsInitRequest;
	uint32_t dwRxOverrun;	/* OUT_BULK ХǡǼ */
	uint8_t bRxMaxpOverrun;	/* OUT_BULK MAXPХΥե饰 */
	uint8_t bEndNumber;
	uint8_t bInactive;	/* set when the ep is halted/request are queued but not executed */
	uint8_t bDma_ena;	/* DMA transfer enable info */
	struct mcd_dma_channel *pDmaChannel;
#if MCD_DEBUG>0
	struct usb_endpoint_descriptor epDescriptor;
#endif
};

struct mcd_gadget {
	struct usb_gadget *pGadget;	/* the gadget */
	struct usb_gadget_driver *pGadgetDriver;	/* it's driver */
	uint8_t bEnd0Stage;
	uint8_t bSuspended;
	uint8_t bIsSelfPowered;
	uint8_t bSetAddress;
	uint8_t bAddress;
	uint8_t bTestMode;
	uint8_t bTestModeValue;
	void *pEnd0Buffer;	/* this is the buffer, clients can change */
	struct dma_pool *pool[HCD_BUFFER_POOLS];

	struct mcd_gep ep[MCD_NUM_EPS];
};

extern struct mcd_gadget *mcd_first_gadget;
extern const uint8_t mcd_test_packet[MCD_TEST_PACKET_SIZE];

extern irqreturn_t mcd_gadget_link_irq(int irq, void *data);
extern void mcd_gadget_start(struct mcd_gadget *mgadget);
extern void mcd_gadget_stop(struct mcd_gadget *mgadget);
extern void mcd_gadget_disconnect(struct mcd_gadget *mgadget);
extern int mcd_gadget_ep0_txstate(struct mcd_gadget *mgadget);
extern int mcd_gadget_ep0_rxstart(struct mcd_gadget *mgadget);
extern int mcd_gadget_ep0_rxstate(struct mcd_gadget *mgadget);
extern void mcd_gadget_flush_ep_requests(struct mcd_gep *ep, const int status);
extern void mcd_gadget_service_tx_avail(struct mcd_gadget *mgadget, uint8_t bEnd);
extern void mcd_gadget_service_rx_ready(struct mcd_gadget *mgadget, uint8_t bEnd);
extern void mcd_gadget_complete_ep0_request(void);
extern int mcd_gadget_queue_length(struct list_head *lh);
extern void mcd_gadget_resume(struct mcd_gadget *mgadget);
extern void mcd_gadget_suspend(struct mcd_gadget *mgadget);
extern void mcd_gadget_disconnect(struct mcd_gadget *mgadget);
extern void mcd_gadget_reset(struct mcd_gadget *mgadget);

static inline struct usb_request *mcd_get_current_request(struct mcd_gep *pEnd)
{
	return (list_empty(&((pEnd)->req_list)) ? NULL :
		list_entry((pEnd)->req_list.next, struct usb_request, list));
}

static inline char *dump_usb_request(struct usb_request *req)
{
	static char buff[256];
	snprintf(buff, sizeof(buff),
		 "req=%p, req->request.length=0x%0x, req->request.zero=0x%x, "
		 "req->request.actual=0x%x, req->request.status=%d", req,
		 req->length, req->zero, req->actual, req->status);
	return buff;
}

static inline int mcd_gadget_find_end(struct usb_ep *pGadgetEnd)
{
	return 1;
}

#endif			/* _USB_MCD_GADGET_H_ */
