/*-----------------------------------------------------------------------------
 * drivers/usb/mcd/usb_mcd_host.c
 * 
 * USB controller driver
 * 
 * Copyright 2009 Sony Corporation
 *
 * This file is part of the Inventra Controller Driver for Linux.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _USB_MCD_HOST_H_
#define _USB_MCD_HOST_H_

#define MCD_HEP_EP0_IDX		0
#define MCD_RESET_TIME		50000 /* us */
#define MCD_RESUME_TIME		20000 /* us */

struct mcd_hep {
	u32 state;
	spinlock_t lock;
	struct mcd_dma_channel *dmach;
	struct urb *urb;
	struct usb_device *dev;
	struct usb_host_endpoint *hep;
	u32 attributes;
};

#define MCD_HEP_ATR_IOREMAP	(1 << 0)
#define MCD_HEP_ATR_NO_DMA	(1 << 1)

struct mcd_port {
	u32 state;
	spinlock_t lock;
	struct usb_port_status status;
};

#define MCD_NUM_HUB_PORT	1

struct mcd_hub {
	struct mcd_port port[MCD_NUM_HUB_PORT];
};

struct mcd_host_ops {
	struct usb_hcd *(*create_hcd)(struct mcd_device *);
	int (*add_hcd)(struct usb_hcd *, unsigned int, unsigned long);
	void (*remove_hcd)(struct usb_hcd *);
	void (*put_hcd)(struct usb_hcd *);
	void (*notify_over_current)(struct mcd_port *);
	irqreturn_t (*link_irq)(int, void *);
};

struct mcd_host {
	struct mcd_device *mdev;

	u8 nep_in;
	u8 nep_out;

	struct mcd_hub hub;
	struct mcd_hep ep0;
	struct mcd_hep ep_in[MCD_NUM_EPS-1];
	struct mcd_hep ep_out[MCD_NUM_EPS-1];
};

/* functions */

static inline struct mcd_host *hcd_to_mhost(struct usb_hcd *hcd)
{
	return (struct mcd_host *)(hcd->hcd_priv);
}

static inline struct usb_hcd *mhost_to_hcd(struct mcd_host *mhost)
{
	return container_of((void *)mhost, struct usb_hcd, hcd_priv);
}

static inline struct mcd_hub *port_to_hub(struct mcd_port *port)
{
	return container_of((void *)port, struct mcd_hub, port);
}

static inline struct mcd_host *hub_to_mhost(struct mcd_hub *hub)
{
	return container_of((void *)hub, struct mcd_host, hub);
}

/* mcd_host */
extern irqreturn_t mcd_host_link_irq(int irq, void *data);

/* mcd_hub */
extern int mcd_hub_status_data (struct usb_hcd *hcd, char *buf);
extern int mcd_hub_control(struct usb_hcd *hcd, u16 typeReq, u16 wValue,
			   u16 wIndex, char *buf, u16 wLength);
extern int mcd_hub_bus_suspend(struct usb_hcd *);
extern int mcd_hub_bus_resume(struct usb_hcd *);
extern void mcd_hub_irq(struct mcd_hub *hub, u8 intrusb);

/* mcd_port */
extern void mcd_port_init(struct mcd_port *port);
extern void mcd_port_notify_over_current(struct mcd_port* port);

/* mcd_hep */
extern void mcd_hep_set_atr(struct mcd_port *port, u8 epadr, u32 attribute);
extern void mcd_hep_clear_atr(struct mcd_port *port, u8 epadr, u32 attribute);
extern void mcd_hep_cntl_init(struct mcd_device *mdev);
extern void mcd_hep_tx_init(struct mcd_device *mdev, u8 index);
extern void mcd_hep_rx_init(struct mcd_device *mdev, u8 index);
extern void mcd_hep_cntl_irq(struct mcd_device *mdev);
extern void mcd_hep_tx_irq(struct mcd_device *mdev, struct mcd_hep *ep);
extern void mcd_hep_rx_irq(struct mcd_device *mdev, struct mcd_hep *ep);
extern int mcd_hep_urb_enqueue(struct usb_hcd *hcd, struct usb_host_endpoint *ep,
			       struct urb *urb, gfp_t mem_flags);
extern int mcd_hep_urb_dequeue(struct usb_hcd *hcd, struct urb *urb);
extern void mcd_hep_endpoint_disable(struct usb_hcd *hcd,
				     struct usb_host_endpoint *ep);

#endif /* _USB_MCD_HOST_H_ */
