/*
 * include/asm-arm/arch-cxd4105/regs-gpio.h
 *
 * CXD4105 address map
 *
 * Copyright 2005,2006 Sony Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 */
#ifndef __REGS_GPIO_H__
#define __REGS_GPIO_H__

#if defined(CONFIG_ARCH_CXD4108)
# define CXD4105_GPIO(x)	(CXD4105_APB2_BASE+SZ_1M*7+0x10000+0x10000*(x))
# define GPIOE_NR		7
# define GPIOSYS_NR		8
#else /* (CXD4105) */
# define CXD4105_GPIO(x)	(CXD4105_APB2_BASE+SZ_1M*7+0x1000+0x100*(x))
# define GPIOSYS_NR		20
#endif
#define CXD4105_GPIOSYS		(CXD4105_GPIO(GPIOSYS_NR))

#define CXD4105_AGPIOSYS	(CXD4105_APB2_BASE+SZ_1M*7+0)

#define VA_GPIO(x)		IO_ADDRESSP(CXD4105_GPIO(x))
#define VA_GPIOSYS		IO_ADDRESSP(CXD4105_GPIOSYS)
#define VA_AGPIOSYS		IO_ADDRESSP(CXD4105_AGPIOSYS)

#if defined(CONFIG_ARCH_CXD4108)
#define CXD4105_OCTRL           (CXD4105_APB2_BASE+SZ_1M*7+0xa0000)
#define OCTRL_DRV2		0x00
#define OCTRL_DRV2SET		0x04
#define OCTRL_DRV2CLR		0x08
#define OCTRL_DRV1		0x0c
#define OCTRL_DRV1SET		0x10
#define OCTRL_DRV1CLR		0x14
#define OCTRL_RESEN		0x18
#define OCTRL_RESENSET		0x1c
#define OCTRL_RESENCLR		0x20
#define OCTRL_PUPD		0x24
#define OCTRL_PUPDSET		0x28
#define OCTRL_PUPDCLR		0x2c
#else /* CXD4105 */
#define CXD4105_OCTRL           (CXD4105_APB2_BASE+SZ_1M*7+0x8000)
#endif
#define VA_OCTRL(x)             (IO_ADDRESSP(CXD4105_OCTRL) + (x)*0x100)

#if defined(CONFIG_ARCH_CXD4108)
#define CXD4105_MISCCTRL(x)	(CXD4105_APB2_BASE+0x7b0000 + (x)*0x100)
#define VA_MISCCTRL(x)		IO_ADDRESSP(CXD4105_MISCCTRL(x))
#define MISCCTRL_DATA		0x00
#define MISCCTRL_SET		0x04
#define MISCCTRL_CLR		0x08

#define MISCCTRL_XRESETREQ	1
#define MISCCTRL_STABLERUN	2

/* XPOWER_OFF  = GPIOE[0] pin = GPIOE bit0 */
#define GPIO_POWEROFF	GPIOE_NR
#define XPOWER_OFF_BIT  0
#define XPOWER_OFF	(1 << XPOWER_OFF_BIT)

#endif /* CONFIG_ARCH_CXD4108 */

#if defined(CONFIG_ARCH_CXD4105)

#ifdef CONFIG_CXD4105_PM2
/* XPOWER_OFF pin */
   /* GPIO2[1] (SSP0_FRM) */
#  define GPIO_POWEROFF	2
#  define XPOWER_OFF    0x00000002
/* STABLE_RUN pin (for reset) */
#  define GPIO_STABLERUN 0
#  define STABLERUN_BIT  15
#  define STABLERUN_MASK (1 << STABLERUN_BIT)
#endif /* CONFIG_CXD4105_PM2 */

#endif /* CONFIG_ARCH_CXD4105 */


/* GPIO registers */
#define GPIO_DIR		0x00
#define GPIO_DATA_READ		0x04
#define GPIO_DATA_SET		0x08
#define GPIO_DATA_CLEAR		0x0c

/* GPIOsys registers */
#define GPIO_INTLS		0x10
#define GPIO_INTHE		0x14
#define GPIO_INTLE		0x18
#define GPIO_INTEN		0x1c
#define GPIO_INTST		0x20	/* read only */
#define GPIO_INTCL		0x24	/* write only */
#define GPIO_INPUTEN		0x28
#define GPIO_RESEN		0x2c
#define GPIO_PUPD		0x30

/* GPIOeasy registers */
#define GPIOE_INEN		0x10
#define GPIOE_PORTSEL(x)	(0x14 + (x)*4)
#define GPIOE_HOLDEN		0x54
#define GPIOE_RESEN		0x58
#define GPIOE_PUPD		0x5c

#ifndef __ASSEMBLY__
# include <asm/io.h>
static inline void gpio_set(int nr, int reg, int bitnr)
{
	u32 dat;

	dat = readl(VA_GPIO(nr)+reg);
	dat |= 1 << bitnr;
	writel(dat, VA_GPIO(nr)+reg);
}

static inline void gpio_clr(int nr, int reg, int bitnr)
{
	u32 dat;

	dat = readl(VA_GPIO(nr)+reg);
	dat &= ~(1 << bitnr);
	writel(dat, VA_GPIO(nr)+reg);
}
#endif /* !__ASSEMBLY__ */

#endif /* __REGS_GPIO_H__ */
