/* 2008-10-21: File added and changed by Sony Corporation */
#ifndef _ASM_ARM_KPROBES_H
#define _ASM_ARM_KPROBES_H
#ifdef __KERNEL__
/*
 *  Kernel Probes (KProbes)
 *  include/asm-arm/kprobes.h
 *
 *  Copyright 2007 Sony Corp.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/types.h>
#include <linux/ptrace.h>

#define  __ARCH_WANT_KPROBES_INSN_SLOT
#define  ARCH_INACTIVE_KPROBE_COUNT 0
#define flush_insn_slot(p)	do { } while (0)

struct kprobe;
struct pt_regs;

typedef u32 kprobe_opcode_t;

#define BREAKPOINT_INSTRUCTION         0xe7f001f8
#define BREAKPOINT_INSTRUCTION_2       0xe7f001fC

#define MAX_INSN_SIZE 2
#define MAX_STACK_SIZE 72

#define MIN_STACK_SIZE(ADDR) (((MAX_STACK_SIZE) < \
        (((unsigned long)current_thread_info()) + THREAD_SIZE - (ADDR))) \
        ? (MAX_STACK_SIZE) \
        : (((unsigned long)current_thread_info()) + THREAD_SIZE - (ADDR)))


#define ARCH_SUPPORTS_KRETPROBES

#define JPROBE_ENTRY(pentry)	(kprobe_opcode_t *)(pentry)

void arch_remove_kprobe(struct kprobe *p);
void kretprobe_trampoline(void);

/* Architecture specific copy of original instruction*/
struct arch_specific_insn {
	/* copy of the original instruction */
	kprobe_opcode_t *insn;
};

struct prev_kprobe {
	struct kprobe *kp;
	unsigned long status;
	unsigned long old_cpsr;
	unsigned long saved_cpsr;
};

/* per-cpu kprobe control block */
struct kprobe_ctlblk {
	unsigned long kprobe_status;
	unsigned long kprobe_old_cpsr;
	unsigned long kprobe_saved_cpsr;
	struct prev_kprobe prev_kprobe;
	long *jprobe_saved_sp;
	struct pt_regs jprobe_saved_regs; /* save ctx as part of jprobe */
	kprobe_opcode_t jprobes_stack[MAX_STACK_SIZE]; /* explicit stack copy */
};

extern int kprobe_exceptions_notify(struct notifier_block *self,
				    unsigned long val, void *data);
extern int kprobe_fault_handler(struct pt_regs *regs, int trapnr);

#endif	/* __KERNEL__ */
#endif	/* _ASM_ARM_KPROBES_H */
