/* 2008-04-24: File added by Sony Corporation */
/*@@ Test Name     : ppgstat-page-slab-001.c
 *@brief
 * Function Tested : Test the "Pages Slab" information in /proc/ppgstat
 *                   entry of the PPGSTAT feature.
 *                   This program will perform allcating the memory using the kernel function
 *                   kmalloc()and freeing the memory when module is removed.
 *
 *@pre
 * Test Technique  : Functional
 * Compilation     : make 
 *                   ppgstat-page-slab-001.ko
 *@criteria
 * In the ppgstat viewer output for ppgstat-page-slab-log1 the Pages in slab  
 * bit will be set as compared to the base log.
 *
 *
 *
 *@note
 *
 */
#include <linux/module.h>  /* Needed by all modules */

char *slab_value = NULL;

int ppgstat_page_slab_init(void)
{
	int alloc_size=1*1024*1024; /* allocating 1 MB Memory*/
 
	printk("%s %d, Module is inserted \n", __FILE__, __LINE__);
	slab_value = kmalloc(alloc_size, GFP_KERNEL);
        if (!slab_value)
	{
		printk("Error in allocating the Memory\n");
		return ENOMEM;
	}       
	return 0;
}
 
void ppgstat_page_slab_exit(void)
{
	printk("%s %d, Module removed ...\n", __FILE__, __LINE__);
	kfree(slab_value);
}

module_init(ppgstat_page_slab_init);
module_exit(ppgstat_page_slab_exit);
 
MODULE_DESCRIPTION("RMM ppgstat test module");
MODULE_LICENSE("GPL");
