#!/bin/sh

# Exit immediately on errors.
set -e

function chmod_x () {
    for f in "$@"; do
	test -f "$f" && chmod ug+x "$f"
    done
}

# For ALP related works.
#   find patches/ -name "*.qsh" | xargs grep -h chmod
EXEC_SCRIPTS="mkkftrun.pl addr2sym kd sym2addr count-inlines func-size bloat-o-meter"
for f in ${EXEC_SCRIPTS} ; do
    chmod_x scripts/${f}
done

# For kprobe tests.
chmod_x samples/kprobes/user-tests/*.sh

# For SS related works.
chmod_x setup-*

# For git to properly work with .pc/ files.
chmod -R ug+rw .pc/

# This script itself
chmod_x scripts/fix-filemodes.sh

# Kernel header scripts
chmod_x scripts/make_kernel_headers.sh
chmod_x scripts/install_kernel_headers.sh
chmod_x scripts/copy_headers.sh

# Kinspect scripts
chmod_x scripts/kinspect/splitter.rb scripts/kinspect/kbootchart.rb
chmod_x scripts/kinspect/dummy_kinspect.rb
chmod_x scripts/kinspect/viewer/httpserver.rb
chmod_x scripts/kinspect/viewer/html/kipct.cgi

# bootchart scripts
chmod_x scripts/bootchart/bootchart.pl scripts/bootchart/script/bootchartd

# Looks at the current index and checks to see if merges or updates
# are needed by checking stat() information.
# This is needed for errornous cg-commit log listing all files
# modified by this file.
#
# git-update-index returns non-zero (indicating some files are changed).
# Ignore it by `|| true', so that the entire script exits with zero.
git-update-index --refresh > /dev/null || true
