#!/bin/sh
#
#  Script to make kernel header
#
#  Copyright 2006-2008 Sony Corporation.
#
set -e # stop on error

DEST=kernel_headers.tar.gz
HEADER=kernel_header.tar
KBUILD=kbuild_tree.tar
FIXUP=.fixup_kernel_headers
WORK=`pwd`/.work

#
# Files and directories required other than KConfig/Makefiles.
#
COPY_DIRS="scripts/"

# COPY_FILES should be synced with install_kernel_headers.sh.
COPY_FILES=".config .arch_name .cross_compile .target_name Module.symvers"

fatal () {
    echo "FATAL: $*" > /dev/tty
    rm -rf ${WORK}
    exit 1
}

usage () {
    echo "usage: $0 [ output_file_name.tar.gz ]" > /dev/tty
    exit 0
}

if [ "$#" -ge 2 ]; then
    usage
fi
if [ $# == 1 ]; then
    DEST=$1
fi

for f in ${COPY_FILES} ${FIXUP}; do
    test -f ${f} || fatal "${f} is not found"
done
for d in ${COPY_DIRS} ; do
    test -d ${d} || fatal "${d} is not found"
done

ARCH=`cat .arch_name`
echo "Creating kernel headers (${DEST}) for ${ARCH}..."

#
# make kernel headers
#
rm -rf ${WORK}
mkdir -p ${WORK}

# copy all header files
tar cf - -C include . | (cd ${WORK} ; tar xf -)

# fixup header files
sh ${FIXUP} ${WORK}

# remove unnecessary directories/files
find ${WORK} -type d \( -name "CVS" -o -name ".svn" \) -print | xargs rm -rf
find ${WORK} -type f -name "*~" -print | xargs rm -rf

# change owner/permission
find ${WORK} -type f -print | xargs chmod 644
find ${WORK} -type d -print | xargs chmod 755

# create tarball
tar cf ${HEADER} -C ${WORK} .
rm -r ${WORK}
test -f ${HEADER}
if [ $? -ne 0 ]; then
    echo
    fatal "Cannot create kernel header"
fi

#
# make kbuild tree
#
mkdir -p ${WORK}
find . -type f \( -name "Makefile*" -o -name "Kconfig*" \) \
             ! \( -path "*.work*" -o -path "*.svn*" -o -path "*.pc*" \) | \
             cpio -o --format=newc --quiet | (cd ${WORK}; cpio -iumd --quiet)
for f in ${COPY_FILES} ; do
    cp ${f} ${WORK}
done
for d in ${COPY_DIRS} ; do
    cp -R ${d} ${WORK}
done

# remove unnecessary directory
rm -rf ${WORK}/Documentation
rm -rf ${WORK}/include
rm -rf ${WORK}/.pc
rm -rf ${WORK}/patches

# remove unnecessary directories/files
find ${WORK} -type d \( -name "CVS" -o -name ".svn" \) -print | xargs rm -rf
find ${WORK} -type f -name "*~" -print | xargs rm -rf

# change permission
find ${WORK} -type d -print | xargs chmod 755

# create tarball
tar cf ${KBUILD} -C ${WORK} .
rm -r ${WORK}
test -f ${KBUILD}
if [ $? -ne 0 ]; then
    echo
    fatal "Cannot create kbuild tree"
fi

#
# make archive file
#
tar cfz ${DEST} ${HEADER} ${KBUILD}
rm ${HEADER} ${KBUILD}

echo "done"
ls -l ${DEST}

exit 0
